<?php

namespace App\Helpers;

use App\Modules\Page;
use App\Modules\Settings;
use App\Modules\Categories;
use App\Modules\Credential;
use App\Modules\Gallery;
use App\Modules\Uploads;
use App\Http\Services\PaymentService;
use App\Http\Services\FieldsService;
use App\Http\Services\CartService;


class HelpersController
{

/**
   * This method will return the front of payment in list
   *
   */

public static function getPaymentFront(){
     $listModules = Self::getSettingsFromJson('_active_modules_');
     $paymentModules = [];
     foreach ($listModules as $v){
       $paymentAdmin = \Wh::config($v, "payment");
       if($paymentAdmin != null)
         $paymentModules[] = $v;
     }

     $data= Credential::select('param', 'enable','title','image','description','instruction','keyp')->
                        where('enable','yes')->
                        whereIn('param', $listModules)->get();

    // Initialize an empty array to store enable descriptions
    $rows = [];
    // Iterate over each parameter
    foreach ($data as $val) {
      $rows[] = (new PaymentService())->executeMakePayment($val->param, $val, 'front');
    }
    return $rows;
 }

  /**
  * Just return logo path
  */
 public static function logo_payment($logo=''){
    return !empty($logo)? url('/public/img')."/".$logo : "";
 }
  /**
  * Convert fields to title fields, like
   *  [1=>"Jhon",2=>"Smith",...]  -> ["Fierst Name"=>"Jhon", "Last Name"=>=>"Smith",...]
   *  1,2 = fields id from database
  */
 public static function toFieldsWithName($data = []){
    return (new FieldsService())->convert($data);
 }
/**
   * When customer open the website the website setup a cookie with a unic id for each separated
   * Will return that unic id what you can use for cart or use your imagination
   */

public static function get_sesion(){
    $cookie  = getCookieAndClean("tmp_user_idd"); //@\Cookie::get('tmp_user_id');
    $rand    = \Wh::get_random(20);
    if(empty($cookie)){
         setcookie("tmp_user_idd", $rand, strtotime( '+100 days' ), "/" );
    }
   return !empty($cookie) ? $cookie : $rand;
}

public static function get_nr_updates(){
    try{
        $data = Settings::whereIn('param', ['_update_cms_','_update_theme_'])->get();
        $nr = 0;
        foreach ($data as $val){
         if($val->value1 != $val->value2) $nr++;
        }
     }catch (\Throwable $e) {}
     
    return ($nr??0) >0 ? '<sup class="updatesSup">'.$nr.'</sup>':'';
}

/**
 * Get applied coupon
 */
 public static function getAppliedCoupon($subtotal = 0) {
   return (new CartService())->getAppliedCoupon($subtotal);
 }


 /**
 *  will return type of rows from table "pages" if you just need in case
 */
 public static function getPagesQuery($type='pages', $enable=null){
    return Page::where('type',$type)->orderBY('sort','asc')->get();
 }
 /**
 *  will return the right tab for an individual tab
 */
public static function rightTab($content='', $product=''){
    if (strpos($content, '[reviews]') !== false) {
        return view('standart.product.reviews',compact('product'));
     }elseif (strpos($content, '[specifications]') !== false) {
        return view('standart.product.specification',compact('product'));
     }elseif (strpos($content, '[descriptions]') !== false) {
        return '<div class="col-md-12">'.\Wh::splitlang($product->text).'</div>';
     }else{
        return '<div class="col-md-12">'.\Wh::splitlang($content).'</div>';
     }
}

 /**
 *   here is all the procces for product tabs
 */
 public static function productTab($product=[]){
    $tabs = self::getPagesQuery('tabs','yes');

    $tabsName='';
    $tabsContent='';
    foreach($tabs as $i=>$valtabs){
        $options = fromJson($valtabs->options??'', false);
       if(($options->enable ?? '')=='enable'){
        $activeTab = $i==0 ? " active":"";

        $tabsName .= '<li class="tab_buttons  Tab'.$valtabs->id.' '.$activeTab.'">
                        <a href="#Tab'.$valtabs->id.'" class="nav-link" onclick="return setTab(\'Tab'.$valtabs->id.'\');" >'.\Wh::splitlang($valtabs->title).'</a>
                      </li> ';

        $tabsContent .= '
                       <div class="tab-pane'.($i==0 ? '':' display_none').'" id="Tab'.$valtabs->id.'">
                         '.\Wh::rightTab($valtabs->text, $product ) .'
                       </div>';
        }
    }
         if(\Wh::UserRole("admin")){
           $tabsName .= '<li class="tab_buttons">
                            <a href="'.url('/admin/page/tabs').'" class="nav-link editTabsAdmin"><i class="fa fa-pen"></i><i class="fa fa-plus"></i></a>
                        </li> ';
            }
       return count($tabs)>0 ? compact('tabsName','tabsContent') : '';
   }


  /**
     * this function will prepare the the data for the variations or specifications,
     * this is when customers chose from the left or right sidebar an options for sorting
     */
 public static function sugesttionsToCheckboxWithUrl($suggestions=[], $idField=''){
     $li = '';
     $getData  =  \Input::all()?? '';
     $arrayValues = isset($getData[$idField]) ? explode("-", (string)$getData[$idField]): [];

     foreach($suggestions as $suggestion){
     $li .= '<label> <input type="checkbox"'. (in_array((string) $suggestion->id, $arrayValues) ? ' checked="" ':'').'
                     value="" onclick="optionsShowProducts(this,\''.$suggestion->id.'\',\''.$idField.'\');"/> '._lang($suggestion->title).'</label> ';
    }

    return $li;
 }


/**
 * will show the right css all depends in what you set in General Settings -> Development Mode
 *
 */

public static function showCss(){
    $css = _CSSSTYELE_;

    foreach($css as $kcss => $vcss){
      if(!empty(mainOption('_developmentMode_'))){
         echo '<link href="'.url($vcss).'?vers='.date("Ymdhis").'" rel="stylesheet"/>';
       }else{
         $file =  @str_replace( ".css" , ".blade.php" ,$vcss);
         echo '<style type="text/css">
                     <!-- ';
         echo   @require base_path(@$file);
         echo '  -->
                    </style>';
      }
    }
 }

 public static function convertValuesOfArrayToJson($array=[]){
    $newArray = [];
    foreach ($array as $key => $value) {
      if (is_array($value)) {
          $isLang = true;
          foreach ($value  as $k=>$v) {
              if(!is_string($k) && !preg_match('/^[a-z]{2}$/', $k))
               $isLang = false;
          }
         $newArray[$key] = $isLang ?  toJson($value) : $value; // Encode only if all sub-keys are two letters
      } else {
        $newArray[$key] = $value;
      }
    }
     return $newArray;
 }

 /**
  * This will strip_tags an array with 3 lavels
  */
 public static function stripTags($val=""){

 return  is_array($val)? array_map(function($v){
                          return Self::stripTags(Self::stripTags($v));
                         }, $val) : trim(strip_tags($val));

 }

/**
 * When you want to save something and need to show an message use this public static function and in your
 * controller set the sesion like
 * session(['status' => _l('Success Saved')]);or
 * session(['status' =>  '<b style="color:red;">Here are some errors with the data</b>' ]);
 */
public static function sucessSavedMsg(){
    $msg='';
     if (session('status')) {
      $msg=' <div class="col-md-12">
              <div class="allertSuccess">
                  '.session('status').'
              </div>
            </div>';
     session(['status' => '']);
      }

    return $msg;
}

 /**
  * Get child categories if child is empty than show the main categories list
  */
 public static function getCatChild(){
     $catStructure = _CATEGORIES_STRUCTURE_??[];
     $mainID       = \Config::get('maincatID');
     $currentID    = \Config::get('catID');

     $mainCPU      = $catStructure[$mainID]['cpu']??'';
     $current_cpu  = $catStructure[$currentID]['cpu']??'';

     $child = \Arr::where($catStructure, function($value, $key) use($mainCPU) {
        $explodeCpu = explode('/',  $value['cpufull']);
        return count($explodeCpu)>1 && in_array($mainCPU, $explodeCpu);
      });

        // if child is empty than show all the main categories
        $child = empty($child) ?  \Arr::where($catStructure, function($value, $key) {
                                        $explodeCpu = explode('/',  $value['cpufull']);
                                        return !\Wh::instring('/',$value['cpufull']) && $value['type'] =='product' ;
                                     }) : $child;

     $li = '';
     foreach ($child as $k => $v){
        $cp = $v['cpu']??'';
        $current = $current_cpu  == $cp ? " class='activeCat'":'';
        $li .= '<li'.$current.'><a href="'._url("/cat/".$v['cpufull']).'">'._lang($v['title']??'').'</a>';
        $child2 = \Arr::where($catStructure, function($value, $key) use($cp) {
                $explodeCpu = explode('/',  $value['cpufull']);
                return count($explodeCpu)>1 && in_array($cp, $explodeCpu)   ;
            });

          if(!empty($child2)){
           $li .='<ul>';
             foreach ($child2 as $k2 => $v2){
                $current2 = $current_cpu  == ($v2['cpu']??'') ? " class='activeCat'":'';
                '<li'.$current2.'><a href="'._url("/cat/".$v2['cpufull']??'').'">'._lang($v2['title']??'').'</a></li>';
             }
           $li .='</ul>';
          }
        $li .='</li>';
      }


   return $li;
 }



 /**
  * Link to quick edit
  * url -  is the linkt to admin panel
  * \Wh::quickEdit('/admin/page/add-edit/pages/20')
  */
 public static function quickEdit($url='',$classFix='editPageLink'){
    return \Wh::UserRole("admin") ?
             ' <a href="'.url($url).'" class="'.$classFix.'"><i class="fa fa-pen"></i></a>' :'';
 }

 /**
  * Balances tags of string using a modified stack.
  */
 public static function force_balance_tags( $text ) {
    $tagstack = array();
    $stacksize = 0;
    $tagqueue = '';
    $newtext = '';
    // Known single-entity/self-closing tags
    $single_tags = array( 'area', 'base', 'basefont', 'br', 'col', 'command', 'embed', 'frame', 'hr', 'img', 'input', 'isindex', 'link', 'meta', 'param', 'source' );
    // Tags that can be immediately nested within themselves
    $nestable_tags = array( 'blockquote', 'div', 'object', 'q', 'span' );

    // WP bug fix for comments - in case you REALLY meant to type '< !--'
    $text = str_replace('< !--', '<    !--', $text);
    // WP bug fix for LOVE <3 (and other situations with '<' before a number)
    $text = preg_replace('#<([0-9]{1})#', '&lt;$1', $text);

    while ( preg_match("/<(\/?[\w:]*)\s*([^>]*)>/", $text, $regex) ) {
        $newtext .= $tagqueue;

        $i = strpos($text, $regex[0]);
        $l = strlen($regex[0]);

        // clear the shifter
        $tagqueue = '';
        // Pop or Push
        if ( isset($regex[1][0]) && '/' == $regex[1][0] ) { // End Tag
            $tag = strtolower(substr($regex[1],1));
            // if too many closing tags
            if ( $stacksize <= 0 ) {
                $tag = '';
                // or close to be safe $tag = '/' . $tag;
            }
            // if stacktop value = tag close value then pop
            elseif ( $tagstack[$stacksize - 1] == $tag ) { // found closing tag
                $tag = '</' . $tag . '>'; // Close Tag
                // Pop
                array_pop( $tagstack );
                $stacksize--;
            } else { // closing tag not at top, search for it
                for ( $j = $stacksize-1; $j >= 0; $j-- ) {
                    if ( $tagstack[$j] == $tag ) {
                    // add tag to tagqueue
                        for ( $k = $stacksize-1; $k >= $j; $k--) {
                            $tagqueue .= '</' . array_pop( $tagstack ) . '>';
                            $stacksize--;
                        }
                        break;
                    }
                }
                $tag = '';
            }
        } else { // Begin Tag
            $tag = strtolower($regex[1]);

            // Tag Cleaning

            // If it's an empty tag "< >", do nothing
            if ( '' == $tag ) {
                // do nothing
            }
            // ElseIf it presents itself as a self-closing tag...
            elseif ( substr( $regex[2], -1 ) == '/' ) {
                // ...but it isn't a known single-entity self-closing tag, then don't let it be treated as such and
                // immediately close it with a closing tag (the tag will encapsulate no text as a result)
                if ( ! in_array( $tag, $single_tags ) )
                    $regex[2] = trim( substr( $regex[2], 0, -1 ) ) . "></$tag";
            }
            // ElseIf it's a known single-entity tag but it doesn't close itself, do so
            elseif ( in_array($tag, $single_tags) ) {
                $regex[2] .= '/';
            }
            // Else it's not a single-entity tag
            else {
                // If the top of the stack is the same as the tag we want to push, close previous tag
                if ( $stacksize > 0 && !in_array($tag, $nestable_tags) && $tagstack[$stacksize - 1] == $tag ) {
                    $tagqueue = '</' . array_pop( $tagstack ) . '>';
                    $stacksize--;
                }
                $stacksize = array_push( $tagstack, $tag );
            }

            // Attributes
            $attributes = $regex[2];
            if ( ! empty( $attributes ) && $attributes[0] != '>' )
                $attributes = ' ' . $attributes;

            $tag = '<' . $tag . $attributes . '>';
            //If already queuing a close tag, then put this tag on, too
            if ( !empty($tagqueue) ) {
                $tagqueue .= $tag;
                $tag = '';
            }
        }
        $newtext .= substr($text, 0, $i) . $tag;
        $text = substr($text, $i + $l);
    }

    // Clear Tag Queue
    $newtext .= $tagqueue;

    // Add Remaining text
    $newtext .= $text;

    // Empty Stack
    while( $x = array_pop($tagstack) )
        $newtext .= '</' . $x . '>'; // Add remaining tags to close

    // WP fix for the bug with HTML comments
    $newtext = str_replace("< !--","<!--",$newtext);
    $newtext = str_replace("<    !--","< !--",$newtext);

    return $newtext;
}

/**
 * This public static function show the first 300 words from text
 */
public static function excerpt($contetnt='',$limit=300) {
  $contetnt =  strip_tags($contetnt,'<p>');
  $excerpt = explode(' ', $contetnt, $limit);
  if (count($excerpt)>=$limit) {
    array_pop($excerpt);
    $excerpt = implode(" ",$excerpt).'...';
  } else {
    $excerpt = implode(" ",$excerpt);
  }
  $excerpt = preg_replace('`\[[^\]]*\]`','',$excerpt);
  return \Wh::force_balance_tags( $excerpt);
}


public static function setVariable(){
    @\Config::set('userRole', @\Wh::UserRole() );
    return "";
 }

/**
  * Return the right routes
  */
public static function sort_root($array=[],$lang=""){
    foreach($array as $k=>$get){

        $is  = is_array($get)? $get[0] : "get";
        $val = is_array($get)? $get[1] : $get;
        $return = "";

        $langSet = LANGW;
        $langMain = LANGMAIN;
        $langIs = $langSet != $langMain ? "/".$langSet : "";

        if(!empty($lang)&& $is=="get"){
          $k = $langIs.$k;
        }
      if(isset($get['where'])){
          \Route::$is($k, $val)->where($get['where'][0],$get['where'][1]);
      }elseif(isset($get['name'])){
          \Route::$is($k, $val)->name($get['name']);
        }else{
          \Route::$is($k, $val);
        }
     }
 }

 /**
  *
  */

  public static function check_col($text=""){
       $return = strpos($text, 'col-md') !== false? $text :
                                 '<div class="col-md-12 inside_grid" id="id'.rand(999999,100000).'">'.$text.'</div>';

      //$return = str_replace('id="id','data-editable data-name="article" id="id',$return);
      return  $return;
  }

  /**
  * This public static function will return Blcok like Footer header
  */

  public static function editableBlocks($conent='',$return = 'content'){
          $val =  explode("~~~~~",$conent);
          $page =  $val[1]??'';
          return $return == 'content' ? \Wh::shortCode(\Wh::splitlang($page)) :
                    view("standart.default.header-css",['optionsData'=> $val[0]??'']);
  }



/**
   * Oance used helper maybe will need for future, for that i keep it
   */
  public static function remove_dublicate($lang=""){
    $contetn = file_get_contents(base_path("/language/".$lang.".json"));
    $json= json_decode($contetn, true );

    $vfinal = [];
    $finalend=[];
    foreach($json as $k=>$v){
        if(!isset($vfinal[trim(strtolower($k))])){
            $vfinal[trim(strtolower($k))] = $v ;
         }
     }

     \File::put(base_path() . '/language/'.$lang.'.json', toJson($vfinal));
     printw($vfinal);
 }
  //remove_dublicate('de');
  //exit();

public static function firstLetterUpercase($word ='', $original= ''){
    $firstLetter = mb_substr(trim($original),0,1, 'UTF-8');
    if(preg_match('/[A-Z]$/',$firstLetter)==true) {

        $first = mb_substr($word,0,1, 'UTF-8');
        $last  = mb_substr($word,1);
        $first = mb_strtoupper($first, 'UTF-8');
        $last  = mb_strtolower($last, 'UTF-8');
        return $first.$last;
    }else{
        return $word;
    }
}



/**
 * Get shipping cost
 * id - shipping id
 * $total - cart total; it will calculate the shipping from total
 */

public static function getShippingCost($id, $total=0) {
   $shipping = \DB::table('shiping')->where('id', $id)->first();
   $price    = \Wh::check_price($shipping->price??'');
   $cost     = $shipping->type_shipping == 3 ? ($total * $price)/100 :
                                                $price;
   $free     = @floatval($shipping->free_delivery);
   $cost     = $free <= $total && $free > 0 ? 0 : $cost;

   return round($cost,2);
 }
/**
 * generate link by page id
 * $id - page id
 */

 public static function get_link_page($id=0){
    $row=\DB::table('page')->whereid($id)->select("cpu","title")->first();

    return !empty($row) ? '<a href="'._url('/page/'.$row->cpu).'">'.\Wh::splitlang($row->title).'</a>':'';
 }

 /**
 * Get shipping cost
 * $id- Shipping id
 * $field - field; like title or cost
 */

public static function getShippingById($id, $field='shipping_name') {
   $shipping = \DB::table('shiping')->where('id', $id)->select($field)->first();
   return $shipping->$field ?? '' ;
 }


 /**
   * Will return the image full path
   * $image - it's aimage from options or just image name
   * $type - thumb or big (full size)
   */
  public static function getImagePath($image="", $type=""){

     if(empty($image))
       return url('/public/img/thumb_noimage.jpg');

     if(strpos($image, '://') !== false){
        return $image;
     }

     $thumb = $type == "big" ? "": "thumb_";
     $imgPath = "";
     if(strpos($image, '/') === false){
        $imgPath = "/public/imgproduct/".$thumb.$image ;
     }else{
        $img = end(explode("/",$image));
        $imgPath = "/public/imgproduct/".str_replace($img, $thumb.$img, $image);
     }

     return  \File::exists(base_path($imgPath)) ? url($imgPath): url("/public/img/thumb_noimage.jpg");
  }


/**
 * Get gallery of the product for admin
 */
public static function getAdminGallery($imagesOrIdPost = null): string
{
    if (empty($imagesOrIdPost)) {
        return '';
    }

    $images = is_numeric($imagesOrIdPost)
        ? Gallery::where('id_post', $imagesOrIdPost)->get()
        : $imagesOrIdPost;

    if (!is_iterable($images)) {
        return ''; // Handle non-iterable input
    }

    $html = '';
    foreach ($images as $key => $image) {
        $thumbnailUrl = \Wh::getImagePath($image->directory); // Assuming Wh::getImagePath returns a URL

        $html .= '<div class="gallery_product">';
        $html .= '<a class="delete_image" href="'. url('admin/remove-image/'.$image->id).'" '
                . 'onclick="event.preventDefault(); access_url(this.href, \'.show_result_gallery\', \'.response_gallery\', \'none\');">' . 'x' . '</a>';
        $html .= '<a href="#"><img src="' . $thumbnailUrl . '"></a>';
        $html .= '</div>';
    }

    $html .= "<div class='clear'></div>";

    return $html;
}


/**
 * return main image for admin
 */

  public static function adminMainImage($img=''){
    $return1= '<input type="hidden" name="image_main" id="input_img" value="'.$img.'" />';
    $return  = '
          <div class="gallery_product" id="main_img">
             <a class="delete_image" href="#" onclick="remove_main_image(\'#main_img\',\'#input_img\');return false;">x</a>
             <a href="#">
               <img src="'.@\Wh::getImagePath($img).'"  />
             </a>
         </div>
         <div class="clear"></div>
         ';
     return !empty($img)?  $return1.$return : $return1;
}

/**
 * Get gallery of the product for admin
 */
 public static function getUploadsByIdPost($idp){
     $files = Uploads::where('id_post', $idp)->orderBy('id', 'asc')->get();

     $return = "";
     foreach ($files as $file){
        $return .= '<div class="col-md-5">'
                      .$file->file_title.
                   '</div>
                    <div class="col-md-5">
                      <a href="'.url("/admin/product/upload").'/'.$file->id.'">'.$file->originalFileName.'</a>
                    </div>
                    <div class="col-md-2">
                    <a class="fa_delete" href="#" onclick="access_url(\''.\URL::to('/').'/admin/product/remove-file/'.$file->id.'\',\'.listOfFiles\', \'.load_iacon\',\'none\');return false;"></a>
                    </div>
                    <div class="height10px"></div>';
     }
   return $return."<div class='clear'></div>";
}


/**
 * Will get any field from product table
 * id-id product
 * $field - title or price any field
 */
public static function getProductField($id="",$field="title"){
    $product= \DB::table('product')
                ->where('id',$id)
                ->select($field)->first();
    return $product->$field??'';
}

/**
 *  Will get all the Comments(Reviews) for the post or product
 * $id_post - product id
 */
public static function get_comments($id_post = 0){
    $comments = \DB::table('comments')
                       ->where('status','=',2)
                       ->where('id_post', $id_post)
                       ->get();
    return $comments;
}

/**
 * Will return the number of commnets in array
 * for eaxample [5=>10, 4=>2, 3=>1, 2=>7, 1=> 5]
 * where array key is stars number and value is the number of comments with key stars
 */
public static function nr_comments($commnets=[]){
     $total = [];
      foreach ($commnets as $commnet){
        if(isset($total[$commnet->stars])){
            $total[$commnet->stars] = $total[$commnet->stars] +1;
        }else{
           $total[$commnet->stars] = 1;
        }
      }
  return $total;
}

/*
Get medium Overall all comments
*/
public static function comments_Overall($commnets=[]){
    $media=0;
    $nr_coments = 0;

     $array_commnts = @\Wh::nr_comments($commnets);

     foreach ($array_commnts  as $k_s=>$v_s){
       $media= $media +  ($k_s *$v_s );
       $nr_coments= $nr_coments + $v_s;
     }
  return !empty($array_commnts)? @round($media / $nr_coments , 1) : 0 ;
}


/*
  Return the Medium Stars by product id
  $id_post- product id
*/
public static function product_stars($id_post=0){
    $media=0;
    $nr_coments = 0;

     $comments = \DB::table('comments')
                       ->where('status','=',2)
                       ->where('id_post', $id_post)
                       ->select('stars')
                       ->get();

   $media = count($comments)>0 ? \Wh::comments_Overall(@$comments):0;

 return count($comments)>0 ? '<span class="stars'.round($media).'"></span>':"";
}

/**
 * Will return the number of commnets in array
 * for eaxample [5=>10, 4=>2, 3=>1, 2=>7, 1=> 5]
 * where array key is stars number and value is the number of comments with key stars
 */
public static function comments_count($status){
     $comments = \DB::table('comments')
                       ->where('status','=',2)
                       ->where('id_post', $id_post)
                       ->get();
    return $comments;
}

 /**
  * Will return true or false if the string is a json
  */
 public static function isJSON($string=''){
   return is_string($string) && is_array(json_decode($string, true)) && (json_last_error() == JSON_ERROR_NONE) ? true : false;
}

/**
 * check if variation exist
 */

public static function check_opt($variation=''){
         $v=json_decode($variation,true);

 return !empty($v['variation']) ? true : false;
}

/**
 * Get Shipping
 */

public static function get_shipping($id=0, $kg=0, $total= 0){
  $order= \DB::table('shiping')
              ->where(function($query) use ($id) {
                  $query->where('country', $id)
                        ->orWhere('country','=',0);
              });
              if(!empty($kg)){
              $order = $order->where(function($query) use ($kg) {
                          $query->where('weight', '>=', $kg)
                                    ->orWhere('weight','=',0);
                          });
              }
              //->where('weight', '>=', $kg)
              $order = $order->where('hide_show','!=','3')
                      ->orderBy('price', 'asc')
                      ->get();

        $result="";
        $checkarray=[];
        $i=0;
        $cost=[];
        //printw($order) ; return '';
        if(count($order)>0){

            foreach($order as $v){
             if(!in_array(strtolower(trim(@$v->shipping_name)), $checkarray) ){
                 $i++;
                 $free= @floatval($v->free_delivery);

                $cost= $v->type_shipping ==3 ? round(($total * $v->price)/100, 2) : \Wh::check_price($v->price, 'none');
                $cost=  @$free <= $total && $free > 0 ? 0 : $cost;

                $price_shipping = $cost > 0 ? priceToHtml($cost, 0, 'not')   : _l("FREE");

                $total_price = $cost + $total;

                $result .= '<li> '
                             .'<label>'
                                   . '<input type="radio" autocomplete="off" onclick="shipping_calculate(\''.priceToHtml($cost,0,'not').'\', \''.priceToHtml($total_price,0,'not').'\');" value="'.$v->id.'" name="deliverymethod" class="cl_deliverymethod" > '
                                   . \Wh::splitlang($v->shipping_name). ' '
                             .'</label><span> '.$price_shipping.'</span></li>';
             }
             $checkarray[] = strtolower(trim(\Wh::splitlang($v->shipping_name)));
         }
        }

      return count($order)>0 ? $result: "" ;

}

public static function shippingCount($countryId = '')
{
    $query = \DB::table('shiping')
        ->where('hide_show', '!=', '3');

    if (!empty($countryId)) {
        $query->where(function ($q) use ($countryId){
            $query->where('country', 0)
                  ->orWhere('country', $countryId);
        });

    } else {
        $query->where('country', 0);
    }

    return $query->count() > 0;
}


public static function return_js($array=[], $data=[], $form="#checkoutform"){
    $js2="";

    foreach ($array as $check ){
        if(empty($data[$check])){
            $type="$form input[name='$check'] , $form select[name='$check'], $form textarea[name='$check']";
            $js2 .= ' $("'.$type.'").css({"outline":"1px solid red", "box-shadow":"0px 0px 4px red","-moz-box-shadow":"0px 0px 4px red","-webkit-box-shadow":"0px 0px 4px red" });  ';
       }
     }
return $js2;
}
/**
 * Get list of countries
 */
public static function list_countries_db($id="",$field=""){
        $country = !empty($id)? \DB::table('shippcountry')->find($id) : \DB::table('shippcountry')->get();
        return !empty($id)? $country->$field : $country ;
}
/**
 * Get country by code
 */
public static function get_countrybycode($code, $field) {
  $country_cost = \DB::table('shippcountry')->where('code', $code)->first();
  return  $country_cost->$field;
 }

/**
 * Get country field by id
 * id- id country
 * field - any field
 */
 public static function getCountryById($id, $field) {
    $country = \DB::table('shippcountry')->find(intval($id)) ;
    return $country->$field ?? '';
}

 /**
  * Get category by title,
  * if title category with this title not exist than insert it
  * may need when import
  * return category id
  */

public static function get_catbytitle($title=0){
   $categories = \DB::table('categories')
                       ->where('title', 'like', '%'.$title.'%')
                       ->first();
   if(empty($categories)){
        $id = \DB::table('categories')->insertGetId( [
              'title' => $title,
              'metad' => $title,
              'metak' => $title,
              'cpu' => str_slug($title)
              ]);
        //echo $id;
   }
    return empty($categories)? $id : $categories->id;
}


/**
  * Get category by field,
  *  return category field
  * $field = cpu
  * $where = id (field name)
  * $equal = 2 (any value)
  * it will be: get 'cpu' where 'id' = '2' select 'cpu'
  *  \Wh::get_catbyfield($field, $where, $equal
  */
public static function get_catbyfield($field, $where, $equal){
   $categories = \DB::table('categories')
                       ->where($where,$equal)
                       ->select($field)
                       ->first();
    return $categories->$field;
}
/**
  * Get categores by list,
  *  return categories list in array
  *  you have tu use in foreach
  */

public static function catList($parent=0, $admin="" ){

   $categories = !empty($parent) && $parent == "cat" ?  \DB::table('categories') : \DB::table('categories')-> where('parent','=',$parent) ;

  if(empty($admin)){
    $categories = $categories -> where('tip','!=',2);
  }
 return $categories->get();
}


public static function getCategoriesByType($type=""){
   return !empty($type) ?
            Categories::where('type', $type)->orderBy("id","asc")->get():
            Categories::orderBy("id","asc")->get();
}


public static function getCategoriesStructure($cat=[], $parent="", $nr= 0, $type=""){
    $return ="";
    $split=" <span style='margin-left:".$nr."px;'>></span> ";
     $nr= $nr + 15;
    $split = $parent==0? "": $split;
    foreach($cat as $maincat){
       if($maincat->parent==$parent){
            $return .=view('admin.layouts.categoryleft', compact("maincat","split","type"));
            $return .= \Wh::getCategoriesStructure($cat, $maincat->id, $nr, $type);
       }
    }
  return $return;
}

/**
 * will return the category for select
 * $cat - categories array
 * $parrent - if you want to be selected
 */
public static function getCategoriesStructureForSelect($catArrayOrtype="", $parent="", $nr= 0, $url="", $is_in=[]){
    $categoriesByType = is_string($catArrayOrtype) ?
                           \Wh::getCategoriesByType($catArrayOrtype) : $catArrayOrtype;
    return self:: getCategoriesStructureForSelectImpl($categoriesByType, $parent, $nr, $url, $is_in) ;
}


public static function getCategoriesStructureForSelectImpl($cat=[], $parent="", $nr= 0, $url="", $is_in=[]){
     $return ="";
     $split="";
     for($i=1;$i<=$nr;$i++){
       $split .= "&nbsp;";
     }
      $nr= $nr + 2;
     //$split = $parent==0? "": $split;
     foreach($cat as $maincat){
       if($maincat->parent==$parent){
             $id_or_url=  strpos($url, 'admin') !== false ? $maincat->id :
                                                            \Wh::generate_cat_url($maincat->id);

             $val= !empty($url) ? $url."/".$id_or_url : $maincat->id;


            $selected = $maincat->id == $is_in? 'selected=""': "";
            $mainCat =  empty($split)? "data-val='main'":"";
            $return .='<option value="'.$val.'" '.$selected.' '.$mainCat.'> '.$split.\Wh::splitlang(@$maincat->title).'</option>';
            $return .= self::getCategoriesStructureForSelectImpl($cat, $maincat->id, $nr, $url, $is_in);
       }
     }
  return $return;
}

/**
 * will return the category in checkbox list
 * $cat - categories array
 * $parrent - if you want to be checked
 */

/**
 * will return the category for select
 * $cat - categories array
 * $parrent - if you want to be selected
 */
public static function getCategoriesStructureCheckbox($catArrayOrtype="", $parent="", $nr= 0,  $isIn=[]){
    $categoriesByType = !is_string($catArrayOrtype) ?
                        $catArrayOrtype : \Wh::getCategoriesByType($catArrayOrtype);

    return self:: getCategoriesStructureCheckboxImpl($categoriesByType, $parent, $nr,  $isIn) ;
}

public static function getCategoriesStructureCheckboxImpl($cat=[], $parent="", $nr= 0, $isin=[]){
     $return ="";
      $nr= $parent == 0 ? 0 : $nr + 7;

     foreach($cat as $maincat){
       if($maincat->parent==$parent){
          $if_class = in_array($maincat->id, $isin) ? " anchor_scroll" : "";
          $if_check = in_array($maincat->id, $isin) ? "checked=''" : "";

            $return .='
                    <label class="product_cat_single'.$if_class.'">
                      <input style="margin-left:'.$nr.'px;" name="cat[]" '.$if_check.' type="checkbox" value="'.$maincat->id.'"> '.\Wh::splitlang($maincat->title).'
                    </label>  ';
            $return .= self::getCategoriesStructureCheckboxImpl($cat, $maincat->id, $nr, $isin);
       }
     }
  return $return;
}

 /**
  get all the sub categories
 **/
 private static function multidimensionalArray($cat, $set=[], $itemId = 0) {
  try {
    $rows = $cat->filter(function ($ch) use ($itemId) { return $ch->parent== $itemId; }) ;
    foreach ($rows as $item) {
       $cpu = array_key_exists($item->parent??0, $set)? $set[$item->parent??0]["cpufull"]."/". $item->cpu??'' : $item->cpu??'';
       $set[$item->id]= ["cpufull" => $cpu,
                         "cpu"     => $item->cpu,
                         "title"   => $item->title,
                         "type"    => $item->type,
                         "image"   => $item->image,
                         "parent"  => $item->parent==null || empty($item->parent) ? 0 : $item->parent,
                         ];

       $haveCh = $cat->filter(function ($ch) use ($item) { return $ch->parent== $item->id; })->toArray();
       if(count($haveCh)>0){
         $set =  $set + self::multidimensionalArray($cat, $set, $item->id) ;
        }
     }
   } catch (\Exception $e) {

      }

   return $set;
 }

 /**
  * this public static function is necesary to mimify the calls to database
  * it will generate for each category or subcategory the URL, so use it when call product
  *
  */
 public static function getCatHierarchicalArray($cat=[]){
   return !empty($cat) ? toJson(self::multidimensionalArray($cat, [], 0)):"";
 }

/**
* all the time forget strpos so i renamed it like instring is easy to remember, right?
* any one know what instring mean
*/
public static function instring($word="", $string=""){
    if( strpos( $string, $word ) !== false) {
        return true;
    }
    return false;
}

 /**
 * when get response from Arr::where it return the full array
 * so we need to get the first key of array what public static functions bellow do
 * return the first key of array
 * for example
 * [ 3 =>["main", 34, 25 ] ]
 * it will return the number 3
 */
public static function getarray_where($array=[]){
     @reset($array);
     return @key($array);
}

public static function getKey_array_where($array=[]){
     @reset($array);
     return @key($array);
}

/**
 * here we return the cat id by cpu and setup the config variable "catID" and "maincatID" with the right ID
 *
 */
public static function getCatParent($cpu='', $ret=''){
    $catStructure = _CATEGORIES_STRUCTURE_;
    $mainid= 0;

    $idCat = \Arr::where($catStructure, function($value, $key) use ($cpu )  {
        if($value['cpu'] == $cpu){
            \Config::set('catID', $key);
         }
        return $value['cpu'] == $cpu && ($value['type']=="product" || $value['type']=='brand');
     });

     $idCat = @\Wh::getarray_where($idCat);

      if(\Wh::instring("/", @$catStructure[$idCat]['cpufull'])){
          $cpuMain = @head(explode("/", $catStructure[$idCat]['cpufull']));

          $idMainCat = \Arr::where($catStructure, function($value, $key) use ($cpuMain )  {
            if($value['cpu'] == $cpuMain){
                \Config::set('maincatID', $key);
             }
            return $value['cpu'] == $cpuMain;
           });

            $idMainCat =\Wh::getarray_where($idMainCat);
        }else{
            \Config::set('maincatID', $idCat);
            $idMainCat = $idCat;
        }

   return empty($ret)? $idCat : compact('idCat', 'idMainCat');
}


public static function generate_cat_url($cat_id=""){
     $arr= _CATEGORIES_STRUCTURE_??[];
     $json = @\Wh::isJSON($cat_id)? @json_decode($cat_id,true) : [$cat_id];
     $config_cat_id =  \Config::get('catID' );

    return !empty($config_cat_id) ? @$arr[$config_cat_id]['cpufull'] :  @$arr[@$json[0]]['cpufull'];
}

public static function getCatFullCpu($catId=""){
     $arr= _CATEGORIES_STRUCTURE_[$catId] ?? [];
    return $arr['cpufull'] ?? '';
}

public static function getCatFullIdsCpu($catId="", $type = "cpu"){
     $arr    = _CATEGORIES_STRUCTURE_ ?? [];
     $fullCpu = $arr[$catId]['cpufull']??'';

     $idPath = [];
     if(!empty($fullCpu)){
       $explode = explode("/", $fullCpu);
       foreach($explode as $cpu){
          $filtered = array_filter($arr, function($value) use ($cpu) {
                        return isset($value["cpu"]) && $value["cpu"] === $cpu;
                      });
          $idPath[]= !empty($filtered) ? array_keys($filtered)[0] : 0;
       }
     }

     return $type=="cpu"? implode("/", $idPath): $idPath;
}

/**
* Will return the full product url
* $cpu- product cpu
* $cat - is a product json
*/
 public static function productUrl($cpu="",$cat=""){
     $langSet = LANGW;
     $langMain = LANGMAIN;
     $langIs = $langSet != $langMain ?  "/".$langSet : "";

   return \URL::to('/').$langIs."/product/".\Wh::generate_cat_url($cat)."/".$cpu;
 }

/**
* Will return the Breadcrumbs of product
* $row - product array
*/
 public static function productBreadcrumbs($row=[]){
    return \Wh::breadcrumbs();
 }

 /**
  * This public static function will create a breadcrumbs based on config variable what you set up on your controler
  */
 public static function breadcrumbs($array=[],$end=""){
      $fullUrl      = '<ul class="breadcrumbNav">
                         <li><a href="'._url("/").'">'._l('Home').'</a></li>';
     if(!empty($array)){
       foreach($array as $k=>$v ){
         $fullUrl .=  '<li><span>/</span> <a href="'._url($k).'">'._l($v).'</a></li>';
        }
       if(!empty($end)){
          $fullUrl .=  '<li><a href="#">'.$end.'</a></li>';
       }
       return $fullUrl. '</ul>';
     }

     $arr          = _CATEGORIES_STRUCTURE_??[];
     $catid        = \Config::get('catID')??'';
     $productTitle = \Config::get('productTitle')??'';
     $page         = \Config::get('page')??'';
     $pageTitle    = \Config::get('pageTitle')??'';
     $search       = \Config::get('search')??'';


    if(!empty($catid)){
     $cpuFull =  $arr[$catid]['cpufull']??'';
     $explode = explode("/",$cpuFull);
      foreach($arr as $k=>$v ){
        $fullUrl .= in_array($v['cpu'], $explode) ? '<li><span>/</span> <a href="'._url("/cat/".$v['cpufull']).'">'.\Wh::splitlang($v['title']).'</a></li>' : '';
      }
     }
     if(!empty($productTitle)){
       //$fullUrl .=  '<li><span>/</span> <a href="#">'.\Wh::splitlang($productTitle??'').'</a></li>';
     }
     if($page == 'page'){
       $fullUrl .=  '<li><span>/</span> <a href="#">'.\Wh::splitlang($pageTitle??'').'</a></li>';
     }
     if($page == 'search'){
        $fullUrl .=  '<li><span>/</span> <a href="'._url('/shop').'">'._l('Shop').'</a></li>';
        $fullUrl .=  '<li><span>/</span> <a href="'._url('/search/products').'?s='.$search.'">'._l('Search').'</a></li>';
        $fullUrl .=  '<li><span>/</span> <a href="'._url('/search/products').'?s='.$search.'">'.$search.'</a></li>';
     }
     if($page == 'cart'){
       $fullUrl .=  '<li><span>/</span> <a href="'._url('/steps/cart').'">'. _l('Cart').'</a></li>';
     }
     if($page == 'checkout'){
       $fullUrl .=  '<li><span>/</span> <a href="'._url('/steps/checkout').'">'. _l('Checkout').'</a></li>';
     }

     if(!empty($end)){
       $fullUrl .=  '<li><a href="#">'.$end.'</a></li>';
     }
     $fullUrl .= '</ul>';

    return $page == 'home' ? '': $fullUrl;
 }



public static function catChildsId($catID){
            $catStructure =  _CATEGORIES_STRUCTURE_ ?? [];
            $arr=[$catID];
            $cpu = $catStructure[$catID]['cpu'];
            foreach($catStructure as $k=>$cat){
               $explod = explode("/",$cat['cpufull']);
               if(in_array($cpu,$explod) && $cat['cpu'] != $cpu && ($cat['type']=='product' || $cat['type']=='brand')){
                  $arr[]=$k;
                }
            }
            return $arr;
 }

public static function get_categoryarr($query){
    foreach ($query as $child1){
       $arr[]= $child1->id;
        foreach ( catList($child1->id) as $child2){
           $arr[]= $child2->id;
        }
    }
   return $arr;
}
 /**
  * Will generate a randon string
  * $n- length
  * $only_letters - only letter not numbers
  */
public static function get_random($n, $only_letters = ""){
  $arr = !empty($only_letters)?
                      array('a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','r','s','t','u','v','x','y','z','A','B','C','D','E','F', 'G','H','I','J','K','L','M','N','O','P','R','S','T','U','V','X','Y','Z')
                      :
                      array('a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','r','s','t','u','v','x','y','z','A','B','C','D','E','F', 'G','H','I','J','K','L','M','N','O','P','R','S','T','U','V','X','Y','Z','1','2','3','4','5','6','7','8','9','0');
   $new = "";
    for($i = 0; $i <$n; $i++){
        $index = rand(0, count($arr) - 1);
        $new.= $arr[$index];
    }
    return $new;
}


public static function url_s(){
  return str_replace(["http://", "https://"], ["",""], url('/'));
}



/**
  * get current user ID if is login
  */
public static function id_user($id="") {
   $idc = empty($id) ? 0 : 9999999987;
   return \Auth::check() ?  \Auth::id() : $idc;
}


/**
  * get user role by id role
  * id- role id
  */
public static function get_role($id){
  $role = \DB::table('role')
                       ->where("id",$id)
                       ->select("role_name")
                       ->first();
    return $role->role_name;
}

/**
  *
  * get current user role admin or editor or none

    \Wh::UserRole(); return admin;

    if curent user role is admin it will return true other cases false

     \Wh::UserRole('admin'); return true or false
*/
public static function UserRole($name=''){

  try{
    $id = \Wh::id_user();
   if($id>0){
         $role = \DB::table('users')
                       ->where("users.id",$id)
                       ->join('role as r','users.user_role', '=', 'r.id' )
                       ->select('r.role_name')
                       ->first();
        if(!empty($name)){
           return !empty($role) && $role->role_name == $name ? true : false;
        }else{
           return !empty($role) ? @$role->role_name : "none";
        }
       }

        return !empty($name)? false : "none";
   } catch (\Exception $e) {
        return !empty($name)? false : "none";
    }

}


public static function currentRole($role=''){
    $current =  @\Config::get('userRole');
    return  $role == $current ? true : false ;
 }

/**
  * get user name or email by user id
  */
public static function getUserInfoById($id, $field="name"){
    $id = $id=="current" ? \Wh::id_user() : $id;
     $role = \DB::table('users')
                   ->select('name','email','unicid')
                   ->where("id",$id)
                   ->first();
    return @$role->$field;
}

/**
  * get list of pages
  */
public static function getPages($type="pages"){
  $pages = \DB::table('page')
                    ->where("type",$type)
                       ->get();

      $array= [];
      foreach($pages as $v){
        $array[]=[$v->id,$v->title];
      }

    return $array;
}




 /**
  * Return field from table setting by id
  */
 public static function returnvalue($id, $field="value"){
     $get = Settings::where('id',$id)->first();
     return  $get?->$field;
 }

/**
* Return field from table setting by param
* $param - is any param(any word)
* $field - value or value1 or value2
* $field2 - if you want to geth botth like $get->$field ."~". $get->$field2 and after use explode("~", $data)
*
*/
  public static function getSettings($param, $field="value"){
    try {
       $get = Settings::where('param',$param)->first();
       return !empty($get)? $get->$field: "";

    } catch (\Exception $e) {
       return "";
    }
 }

 /**
  * Return field from table setting by param
  *  $param - is any param(any word)
  *  $field - value or value1 or value2
  */
  public static function getSettingsFromJson($param, $field="value1" ){
   $get = Settings::where('param',$param)->first();
   return !empty($get)? fromJson($get->$field??'') : [];
 }


/**
 * Will return value from array by key
 */
public static function getSettingsByKey($param, $key=""){
   $settings = \Wh::getSettingsFromJson($param, "value1");
   return $settings[$key]??'';
 }

 /**
 * Will return value from _CONSTANT_ array by key
 */
public static function constant_key($constant="", $key=""){
   return isset($constant[$key]) ? $constant[$key] : "";
 }

 /**
 * Will return value from _CONSTANT_ array by key
 */
public static function json_key($json="", $key=""){
    $constant = \Wh::isJson($json)? fromJson($json): $json;
   return $constant[$key] ?? "";
 }


 /**
  * Return loop data from table setting by param
  *
  */
  public static function getAllSettings($param){
     return  Settings::where('param',$param)->get();
 }

  /**
  * Update or insert if param is not existing
  * $param - is the parametr where you can recognise your data
  * $value  - varchar(255)
  * $value1 - text accept big data
  * $value2 - varchar(255)
  */
  public static function updateSettings($param, $value = "", $value1 = "", $value2 = "", $autoload = ""){
    Settings::updateOrCreate( 
        ['param'    => $param],
        ['value'    => $value, 
         'value1'   => toJson($value1), 
         'value2'   => $value2, 
         'autoload' => $autoload
        ]);
 }


    /**
  * Update or insert if id is not existing
  * $id -  id settings
  * $value  - varchar(255)
  * $value1 - text accept big data
  * $value2 - varchar(255)
  */
  public static function updateSettingsID($id, $value="", $value1="", $value2=""){
     Settings::updateOrCreate( 
        ['id'=>$id],
        ['value' => $value, 
         'value1' => toJson($value1), 
         'value2' => $value2
        ]);
  }


 public static function returnsize($s,$typ_size=""){
     $eu=["4"=>"32", "6"=>"34", "8"=>"36", "10"=>"38", "12"=>"40", "14"=>"42", "16"=>"46", "18"=>"48", "20"=>"50", "22"=>"52", "24"=>"54", "26"=>"56"];
     $us=["4"=>"1", "6"=>"2", "8"=>"4", "10"=>"6", "12"=>"8", "14"=>"10", "16"=>"12", "18"=>"14", "20"=>"16", "22"=>"18", "24"=>"20", "26"=>"22"];

     $eu_shos=["3"=>"35.5","3.5"=>"36", "4"=>"37", "4.5"=>"37.5", "5"=>"38", "5.5"=>"38.5", "6"=>"39", "6.5"=>"40",
               "7"=>"40.5", "7.5"=>"41", "8"=>"42", "8.5"=>"42.5","9"=>"43", "9.5"=>"43.5"];

     $us_shos=["3"=>"5.5","3.5"=>"6", "4"=>"6.5", "4.5"=>"7", "5"=>"7.5", "5.5"=>"8", "6"=>"8.5", "6.5"=>"9",
               "7"=>"9.5", "7.5"=>"10", "8"=>"10.5", "8.5"=>"11","9"=>"11.5", "9.5"=>"12"];

      if(is_numeric($s)){
        $size =empty($typ_size)? "UK".$s." - EU".$eu[$s]." - US".$us[$s] : "UK".$s." - EU".$eu_shos[$s]." - US".$us_shos[$s];
      }else{
        $size =$s;
      }

   return $size ;
 }


  /**
  * will return the current currency icon
  */
public static function currency(){
      return CURENCY_CODE;
}

 /**
  * get price with currency icon
  * $price - normal price
  * $sale_price - sale price
  * $type - if it's empty than will return the price + incrise what you setup on
  *         Setting page link:  yourwebsite.domain/admin/general-settings/
  * if is not empty will return a clear price.
  */

public static function priceToHtml($price, $salePrice = 0, $type = '')
{
    $currencySymbol     = \Wh::currency();

    if (empty($type)) {
        $price          = \Wh::check_price($price);
        $salePrice      = \Wh::check_price($salePrice);
    }

    $formattedPrice     = "<span>".number_format($price, 2)."</span>";
    $formattedSalePrice = "<span>".number_format($salePrice, 2)."</span>";

    $position = CURENCY_POSITION;
    switch ($position) {
        case "left with space":
            $price_return = $salePrice > 0 ? "<del class='pricesale'>{$currencySymbol} {$formattedPrice}</del> {$currencySymbol} {$formattedSalePrice}":
                                             "{$currencySymbol} {$formattedPrice}";
            break;
        case "right with space":
            $price_return = $salePrice > 0 ? "<del class='pricesale'>{$formattedPrice} {$currencySymbol}</del> {$formattedSalePrice} {$currencySymbol}":
                                             "{$formattedPrice} {$currencySymbol}";
            break;
        case "right":
            $price_return = $salePrice > 0 ? "<del class='pricesale'>{$formattedPrice}{$currencySymbol}</del> {$formattedSalePrice}{$currencySymbol}":
                                             "{$formattedPrice}{$currencySymbol}";
            break;
        default:
            $price_return = $salePrice > 0 ? "<del class='pricesale'>{$currencySymbol}{$formattedPrice}</del> {$currencySymbol}{$formattedSalePrice}":
                                             "{$currencySymbol}{$formattedPrice}";
            break;
    }

    return $price_return;
}

 /**
  * get price only number
  * $price - normal price
  * $sale_price - sale price
  * $price_option - if it's empty than will return the price + incrise what you setup on
  *         Setting page link:  yourwebsite.domain/admin/general-settings/
  * if is not empty will return a clear price.
  */

public static function get_price_number($price=0, $sale_price=0, $price_option=0){

    $price_option = floatval($price_option);

    if($price_option > 0){
       return \Wh::check_price($price_option);
    }else if(($price_option === 0 || $price_option === 0.00) && $sale_price > 0 ){

         return \Wh::check_price($sale_price);
    }else{
        return \Wh::check_price($price);
    }
 }

public static function check_price($price=0, $none=""){
   $rate            = CURENCY_RATE;
   $currency        = !empty($rate) ? floatval($rate): 1;
   $addition_price  = mainOption("_additional_price");
   $additional_type = mainOption("_additional_type");

   $addToPrice = !empty($addition_price)? floatval($addition_price) : 0;

   $price1=  $addToPrice >0 && $additional_type == 'fix' ?
                               round (($price + $addToPrice) * $currency , 2):
                               round (($price + ($price * $addToPrice)/100) * $currency , 2);

   return !empty($none)? round ($price * $currency, 2):  $price1;
}


public static function prepareMenu($data=""){
    $getCurrentUrl= (string) request()->path();
    $data = \Wh::splitlang($data);
    $menu = str_replace(['<a href="#" class="fa_delete fa_small delete_ittem"></a>',
                           '<a href="#" class="fa_edit fa_small"></a>',
                           '<a href="#" class="fa_move move_bdn"></a>',
                           '<a href="#" class="fa_move move_bdn ui-sortable-handle"></a>',
                           'class="simple_link"',
                           'style="opacity: 1;"',
                           ' ui-sortable-handle'
                            ], "", $data);
      $menu= str_replace('<ul>','<ul class="sub-menu">', $menu);

       $langIs = \Wh::returnLang();
       $url_web = \URL::to('/');
       $menu= str_replace('href="/','href="'.$langIs.'/', $menu);
       $menu= str_replace('href="'.$url_web,'href="'.$url_web.$langIs.'/', $menu);

       if($getCurrentUrl !="/"){
          $menu= str_replace($getCurrentUrl.'"', $getCurrentUrl.'" class="active-menu"', $menu);
       }else{
          $menu= str_replace(['"/"','"'.url("/").'"'], ['"'.$getCurrentUrl.'" class="active-menu"'], $menu);
       }
return $menu;
}

/**
 * Get menue
 * $value2 - top_menu or any what you selected
 */

public static function get_menu($value2="none"){
    $get = Settings::where('value2', $value2)->first();
    $menu= !empty($get)? \Wh::prepareMenu($get->value1) : "";
    return $menu;
}

/**
  * This public static function will add new ittem to admin menu
  */
 public static function doAdminMenu(){
    $adminmenu = \Wh::getAllConfigs("adminmenu");

    $menu= ADMIN_MENU;
    if(!empty($adminmenu)){
      foreach($adminmenu as $k=>$v){
        foreach($v as $k1=>$v1){
          $menu[$k1] = isset($menu[$k1]) ? array_merge($menu[$k1], $v1) : $v1;
        }
      }
    }
    ksort($menu);

    // remouve the menus
    $removeMenu = \Wh::config(_THEME_NAME_, "removeAdminMenu");
    if(!empty($removeMenu)){
        foreach($removeMenu as $remove){
            unset($menu[$remove]);
        }
    }

    return $menu;
 }


/**
 * This public static function will generate a list from array
 * ADMIN_MENU - is a constant from array_settings_admin.php
 */
public static function generateAdminMenu($k=[], $type = ''){
     $return = "";
     $global_menu= \Wh::doAdminMenu();
     $is_in= 0;
     $getCurrentUrl= (string) request()->path();

       if(!empty($type)){
             foreach($global_menu as $k_is=>$v){
               foreach($v as $k1=>$v1){

                    if(@\Wh::instring(ltrim($k1,"/"), $getCurrentUrl) !== false && $k1 !="/admin"){
                        $is_in = $k_is ;
                        break;
                      }
                  }
             }
         }

     $menu = !empty($type)? @$global_menu[$is_in] : $k;

     unset($menu['#']);
       if(is_array($menu)){
         foreach($menu as $k_sub=>$v_sub){
            if($v_sub !="hidden"){
                $return .=   '<li>
                                 <a href="'.url($k_sub).'">
                                     '.$v_sub.'
                                 </a>
                           </li>';
                 }
             }
         }

    return $return;
}

/**
 *  this method is used when update the page or category so make sure that
 *  the changes are made also in menu
 * @param type $oldCpu
 * @param type $newCpu
 * @param type $oldTitle
 * @param type $newTitle
 */
public static function updateMenu($oldCpu='', $newCpu='', $oldTitle='', $newTitle=''){
     // Check if $newTitle is an array or an object
    if (is_array($newTitle) || is_object($newTitle)) {
       foreach($newTitle as $lang => $title){
         $oldTitle = fromJson($oldTitle);
         if (!empty($oldTitle[$lang])) {
            \Wh::replaceMenuUpdate($oldCpu, $newCpu, $oldTitle[$lang], $title);
         }
       }
    }else{
        \Wh::replaceMenuUpdate($oldCpu, $newCpu, $oldTitle, $title);
    }

}

/**
 * This will replace data in menu when update page or categories
 */
public static function replaceMenuUpdate($cpu='', $newCpu='', $title='', $newTitle=''){
              $menu = \Wh::getAllSettings('_website_menu_');
           foreach($menu as $m){


            if(\Wh::isJson($m->value1)){
               $json_decode = json_decode($m->value1, true);
               $newArray = [];
               foreach($json_decode as $kk => $vv){
                 $newArray[$kk] =  \Wh::replaceMenuUpdateByVal($vv, $cpu, $newCpu, $title, $newTitle);
               }
               $ittem = toJson($newArray);
             }else{
               $ittem = \Wh::replaceMenuUpdateByVal($m->value1, $cpu, $newCpu, $title, $newTitle);
             }

            \Wh::updateSettingsID($m->id, $m->value, $ittem, $m->value2);
        }
}

/**
*   Abstract method of replaceMenuUpdate
*/
private static function replaceMenuUpdateByVal($val = "", $cpu='', $newCpu='', $title='', $newTitle){
    $ittem = !empty($cpu) && !empty($newCpu) ?
                      str_replace("/".trim($cpu),"/".trim($newCpu), htmlspecialchars_decode($val) ) : $val;

    $ittem = !empty($title) && !empty($newTitle) ?
              str_replace(">".trim(htmlspecialchars_decode($title))."<",">".trim(htmlspecialchars_decode($newTitle))."<", htmlspecialchars_decode($val) ) :
                                            $val;
 return $ittem;
}


/**
 * This is a simple helper check if class and method in the class exist
 * if exists then return the method
 */
public static function executeMethodFromString($string="", $data=[] ){
    $splipClassAndMethod = explode('@',$string);
    $fullClassName       = $splipClassAndMethod[0]??'';
    $functionName        = $splipClassAndMethod[1]??'';

     $return = null;

    // Check if $className is not empty and is a string
    if (!empty($fullClassName) && is_string($fullClassName)) {

        // Check if the class exists
        if (class_exists($fullClassName)) {
            // Instantiate the class
            $instanceObj = new $fullClassName;

            // Check if the object was successfully created
            if (is_object($instanceObj)) {
                // Check if the '$functionName' method exists in the class
                if (method_exists($instanceObj, $functionName)) {
                    // Call the $functionName method dynamically
                    $return = !empty($data) ? $instanceObj->{$functionName}($data) : $instanceObj->{$functionName}();
                }else{
                    $return = $instanceObj;
                }
            }
        }
    }

    return $return;
 }

public static function simple_check($data = "", $name = "", $type = 'module', $vars = []) {
    $path = '';

    switch ($type) {
        case "module":
            if ($name != "theme") {
                $path = "Platform\Modules\\" . ucfirst($name) . "\\" . $data;
            } else {
                $path = "Platform\Themes\\" . _THEME_NAME_ . "\\" . $data;
            }
            break;
        case "cms":
            $path = "App\Http\\" . $data;
            break;
        default:
            // Handle unexpected type
            break;
    }

   return contains($data, '@') ? self::executeMethodFromString($path, $vars) : $data;
}


/**
 * Simple hook public static function
 * in your theme in blade or controller you can add many hooks and from config.php you can add on that part of the page any code you want
 */

public static function hooks($hook = '', $data = []) {
    // Get all hook configurations
    $hooks = self::getAllConfigs("hooks");

    $return = '';

    // Check if there are any hooks configured
    if (!empty($hooks)) {
        // Iterate through each hook
        foreach ($hooks as $type => $hookConfig) {
            // Check if the hook has a configuration for the specified hook
            if (isset($hookConfig[$hook])) {
                // Execute the hook functionality defined by the module
                $return .= \Wh::simple_check($hookConfig[$hook], $type, 'module', $data);
            }
        }
    }

    return $return;
}


 /**
 * Simple Widgets public static function
 *
 */
public static function _widgets($widget=''){
    $widgets = self::getAllConfigs("widgets");

    $return = '';

    // Check if there are any hooks configured
    if (!empty($widgets)) {
        // Iterate through each hook
        foreach ($widgets as $type => $widgetConfig) {
            // Check if the hook has a configuration for the specified hook
            if (isset($widgetConfig[$widget])) {
                // Execute the hook functionality defined by the module
                $return .= \Wh::simple_check($widgetConfig[$widget], $type);
            }
        }
    }

    return $return;
 }


 /**
  * Return the value by key from config.php of all active modules and themes
  *
  * if the key in in config is setted a string value like:
  * 'description'=> 'Asimple theme description'
  * $return ["moduleName"=>"Asimple theme description", "theme"=>"...",...]
  * if the key in in config is setted a array value like
  * 'customLinks'=> [
           '/user/profile' => _l('View Profile'),
           '#showChat'     => _l('Messages'),
      ],
  *
  * $return ["moduleName"=>[
                            '/user/profile' => _l('View Profile'),
                            '#showChat'     => _l('Messages'),
                           ],
             "theme"     =>[...],
  *          ...]
  *
  */
 public static function getAllConfigs($key = "") {
    $activeModules = _ACTIVE_MODULES_;
    $return = [];

    // Retrieve configurations from active modules
    foreach ($activeModules as $moduleName) {
        $moduleConfig = self::config($moduleName, $key);
        if (!empty($moduleConfig)) {
            $return[$moduleName] = $moduleConfig;
        }
    }

    // Retrieve configurations from active theme
    $themeConfig = self::config(_THEME_NAME_, $key);
    if (!empty($themeConfig)) {
        $return['theme'] = $themeConfig;
    }

    return $return;
}

public static function mergeAllConfig($constant = [], $config = '') {
    // Retrieve configurations from dynamic sources
    $dynamicConfigs = Self::getAllConfigs($config);

    // Merge dynamic configurations into the constant array
    if (!empty($dynamicConfigs)) {
        foreach ($dynamicConfigs as $key => $value) {
            if (is_array($value)) {
                foreach ($value as $k => $v) {
                    // Merge dynamic configuration into constant array
                    $constant[$k] = $v;
                }
            }
        }
    }

    return $constant;
}


/**
 * Read config file from theme or module
 */

public static function config($module = "", $field = 'description')
{
    $pathTheme = base_path('platform/Themes/' . $module . '/config.php');
    $pathModule = base_path('platform/Modules/' . $module . '/config.php');

    // Check if the theme config file exists
    if (\File::exists($pathTheme)) {
        $config = require $pathTheme;
    } elseif (\File::exists($pathModule)) { // If theme config doesn't exist, check module config
        $config = require $pathModule;
    } else { // Neither theme nor module config file exists
        return null; // or throw an exception, depending on your requirements
    }

    $param = $config[$field] ?? null;
    // Return the value of the specified field from the config array
    return empty($param) ? null : $param; // Return null if the field doesn't exist
}

 /**
 * This is a short code public static function if you need to use the short code in page
 *
 */
public static function shortCode($text = ''){
    $text= _lang($text);
     //$activate_plugins = _ACTIVE_MODULES_;
    $module = Self::getAllConfigs("shortcode");
    $default = config("app.shortcode");

  if(!empty($module) || !empty($default)){

    $regex = "/\[(.*?)\]/";
    preg_match_all($regex, $text, $matches);

    if(count($matches[1])>0) {
      for($i = 0; $i < count($matches[1]); $i++){

        $match = $matches[1][$i]??'';
        $short_code =trim($match);
        $data=[];

        if (strpos($match, '=') !== false) {
            $array = explode(' ', $match);
            $short_code= trim($array[0]??'');

            unset($array[0]);
            foreach($array as $v){
                $split = explode("=", $v);
                if(empty($split[1])){
                    end($data);         // move the internal pointer to the end of the array
                    $key = key($data);  // fetches the key of the element pointed to by the internal pointer
                    $data[$key] = ($data[$key]??'')." ". ($split[0]??''); //@$split[1];

                }else{
                    $data[($split[0]??'')] = ($split[1]??'');
                }
            }
        }

            // module and theme check
             if(!empty($module)){
                foreach($module as $k=>$v){
                   if(isset($v[$short_code])){
                     $return =\Wh::simple_check($v[$short_code], $k,'module', $data);
                     $text = str_replace("[".$match."]", $return, $text);
                   }
                }
             }

            // deault
            if(!empty($default) && isset($default[$short_code])){
              $return =\Wh::simple_check($default[$short_code], '', 'cms', $data);
              $text = str_replace("[".$match."]", $return, $text);
            }
      }
    }
  }

return $text;
}


 /**
  * will return the logo from the settings page
  */
 public static function _logo(){
    $logo = mainOption("_logo_");

    return !empty($logo)? '<a href="'._url("/").'"><img class="logo" src="'.url('/public/img/'.$logo).'"/></a>' : '';
 }

  /**
  * will return the favicon from the settings page
  */
 public static function _favicon(){
    $favicon = mainOption("_favicon_");
    return !empty($favicon)? '<link rel="shortcut icon" type="image/png" href="'.url('/public/img/'.$favicon).'"/>' : '';
 }


public static function checkEmpty($check="",$before="",$after=""){
    return !empty($check) || $check==="0" || $check===0 ? $before.$check.$after : '';
}



/**
 * Crop image easy and eficient
 */

public static function resize_crop_image($max_width, $max_height, $source_file, $dst_dir, $quality = 80, $crop=true){
    $imgsize = getimagesize($source_file);
    $width = $imgsize[0];
    $height = $imgsize[1];
    $mime = $imgsize['mime'];

    switch($mime){
        case 'image/gif':
            $image_create = "imagecreatefromgif";
            $image = "imagegif";
            break;

        case 'image/png':
            $image_create = "imagecreatefrompng";
            $image = "imagepng";
            $quality = 7;
            break;

        case 'image/jpeg':
            $image_create = "imagecreatefromjpeg";
            $image = "imagejpeg";
            $quality = $quality;
            break;

        default:
            return false;
            break;
    }


    $width_new = $height * $max_width / $max_height;
    $height_new = $width * $max_height / $max_width;
    $r = $width / $height;
    $h_point =0;
    $w_point=0;
    $w_last = $width;
    $h_last = $height;

     if($crop){
    //if the new width is greater than the actual width of the image, then the height is too large and the rest cut off, or vice versa
    if($width_new > $width){
        //cut point by height
        $h_point = (($height - $height_new) / 2);

        $h_last = $height_new;

    }else{
        //cut point by width
        $w_point = (($width - $width_new) / 2);
        $w_last = $width_new;
     }

    }else{
        // No crop
         if ($max_width/$max_height > $r) {
            $max_width = $max_height*$r;
         } else {
            $max_height = $max_width/$r;
          }
     }


    $dst_img = imagecreatetruecolor($max_width, $max_height);
    $src_img = $image_create($source_file);

    //copy image
    imagecopyresampled($dst_img, $src_img, 0, 0, $w_point, $h_point, $max_width, $max_height, $w_last, $h_last);

    $image($dst_img, $dst_dir, $quality);

    if($dst_img)imagedestroy($dst_img);
    if($src_img)imagedestroy($src_img);
}

public static function resize($source_image, $destination, $tn_w, $tn_h, $quality = 95, $wmsource = false)
{
    $info = getimagesize($source_image);
    $imgtype = image_type_to_mime_type($info[2]);

    #assuming the mime type is correct
    switch ($imgtype) {
        case 'image/jpeg':
            $source = imagecreatefromjpeg($source_image);
            break;
        case 'image/gif':
            $source = imagecreatefromgif($source_image);
            break;
        case 'image/png':
            $source = imagecreatefrompng($source_image);
            break;
        default:
            die('Invalid image type.');
    }

    #Figure out the dimensions of the image and the dimensions of the desired thumbnail
    $src_w = imagesx($source);
    $src_h = imagesy($source);


    #Do some math to figure out which way we'll need to crop the image
    #to get it proportional to the new size, then crop or adjust as needed

    $x_ratio = $tn_w / $src_w;
    $y_ratio = $tn_h / $src_h;

    if (($src_w <= $tn_w) && ($src_h <= $tn_h)) {
        $new_w = $src_w;
        $new_h = $src_h;
    } elseif (($x_ratio * $src_h) < $tn_h) {
        $new_h = ceil($x_ratio * $src_h);
        $new_w = $tn_w;
    } else {
        $new_w = ceil($y_ratio * $src_w);
        $new_h = $tn_h;
    }

    $newpic = imagecreatetruecolor(round($new_w), round($new_h));
    imagecopyresampled($newpic, $source, 0, 0, 0, 0, $new_w, $new_h, $src_w, $src_h);
    $final = imagecreatetruecolor($tn_w, $tn_h);
    $backgroundColor = imagecolorallocate($final, 255, 255, 255);
    imagefill($final, 0, 0, $backgroundColor);
    //imagecopyresampled($final, $newpic, 0, 0, ($x_mid - ($tn_w / 2)), ($y_mid - ($tn_h / 2)), $tn_w, $tn_h, $tn_w, $tn_h);
    imagecopy($final, $newpic, (($tn_w - $new_w)/ 2), (($tn_h - $new_h) / 2), 0, 0, $new_w, $new_h);

    #if we need to add a watermark
    if ($wmsource) {
        #find out what type of image the watermark is
        $info    = getimagesize($wmsource);
        $imgtype = image_type_to_mime_type($info[2]);

        #assuming the mime type is correct
        switch ($imgtype) {
            case 'image/jpeg':
                $watermark = imagecreatefromjpeg($wmsource);
                break;
            case 'image/gif':
                $watermark = imagecreatefromgif($wmsource);
                break;
            case 'image/png':
                $watermark = imagecreatefrompng($wmsource);
                break;
            default:
                die('Invalid watermark type.');
        }

        #if we're adding a watermark, figure out the size of the watermark
        #and then place the watermark image on the bottom right of the image
        $wm_w = imagesx($watermark);
        $wm_h = imagesy($watermark);
        imagecopy($final, $watermark, $tn_w - $wm_w, $tn_h - $wm_h, 0, 0, $tn_w, $tn_h);

    }
    if (imagejpeg($final, $destination, $quality)) {
        return true;
    }
    return false;
}


public static function clearUrl($url=''){

    $input = trim($url, '/');
     // If scheme not included, prepend it
    if (!preg_match('#^http(s)?://#', $input)) {
        $input = 'http://' . $input;
    }
    $urlParts = parse_url($input);

    // remove www
    $domain = preg_replace('/^www\./', '', $urlParts['host']);
    return $domain;
}

public static function  AnaliticsURL(){
    $userAgent1 = (string) request()->server('HTTP_USER_AGENT','');
    $userAgent2 = (string) request()->header('User-Agent','');
    $userAgent  = !empty($userAgent2) ? @urlencode($userAgent2) : @urlencode($userAgent1);

    $referer    = urlencode((string) request()->path());
    $url        = urlencode(@str_replace(\URL('/'), '', \URL::previous()));
    return url("/analitic/run")."?userAgent=".$userAgent."&referer=".$referer."&previos=".$url ;
}


/**
 * it will call an external url by curl
 */


public static function get_by_curl($url, $no_return = "") {
        $user_agent='Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36';

    $options = array(
        CURLOPT_RETURNTRANSFER => true,     // return web page
        CURLOPT_HEADER         => false,    // don't return headers
        CURLOPT_FOLLOWLOCATION => true,     // follow redirects
        CURLOPT_ENCODING       => "",       // handle all encodings
        CURLOPT_USERAGENT      => $user_agent, // who am i
        CURLOPT_AUTOREFERER    => true,     // set referer on redirect
        CURLOPT_CONNECTTIMEOUT => 120,      // timeout on connect
        CURLOPT_TIMEOUT        => 120,      // timeout on response
        CURLOPT_MAXREDIRS      => 10,       // stop after 10 redirects
        CURLOPT_SSL_VERIFYPEER => false     // Disabled SSL Cert checks
    );
  try{
    $ch      = curl_init( $url );
    curl_setopt_array( $ch, $options );
    $content = curl_exec( $ch );
    $err     = curl_errno( $ch );
    $errmsg  = curl_error( $ch );
    $header  = curl_getinfo( $ch );
    curl_close( $ch );

    $header['errno']   = $err;
    $header['errmsg']  = $errmsg;
    $header['content'] = $content;
    return !empty($no_return)? "" : $content;
  }catch(Exception $e){
    return "";
  }

 }


/**
 * ----------------------------------------------------------------------------------------
 * Based on `https://github.com/mecha-cms/mecha-cms/blob/master/engine/plug/converter.php`
 * ----------------------------------------------------------------------------------------
 */
// HTML Minifier
public static function minify_html($input) {
    if(trim($input) === "") return $input;
    // Remove extra white-space(s) between HTML attribute(s)
    $input = preg_replace_callback('#<([^\/\s<>!]+)(?:\s+([^<>]*?)\s*|\s*)(\/?)>#s', function($matches) {
        return '<' . $matches[1] . preg_replace('#([^\s=]+)(\=([\'"]?)(.*?)\3)?(\s+|$)#s', ' $1$2', $matches[2]) . $matches[3] . '>';
    }, str_replace("\r", "", $input));
    // Minify inline CSS declaration(s)
    if(strpos($input, ' style=') !== false) {
        $input = preg_replace_callback('#<([^<]+?)\s+style=([\'"])(.*?)\2(?=[\/\s>])#s', function($matches) {
            return '<' . $matches[1] . ' style=' . $matches[2] . \Wh::minify_css($matches[3]) . $matches[2];
        }, $input);
    }
   /*
    if(strpos($input, '</style>') !== false) {
      $input = preg_replace_callback('#<style(.*?)>(.*?)</style>#is', function($matches) {
        return '<style' . $matches[1] .'>'. \Wh::minify_css($matches[2]) . '</style>';
      }, $input);
    }
    if(strpos($input, '</script>') !== false) {
      $input = preg_replace_callback('#<script(.*?)>(.*?)</script>#is', function($matches) {
        return '<script' . $matches[1] .'> '. minify_js($matches[2]) . ' </script>';
      }, $input);
    }
    */
    return preg_replace(
        array(
            // t = text
            // o = tag open
            // c = tag close
            // Keep important white-space(s) after self-closing HTML tag(s)
            '#<(img|input)(>| .*?>)#s',
            // Remove a line break and two or more white-space(s) between tag(s)
            '#(<!--.*?-->)|(>)(?:\n*|\s{2,})(<)|^\s*|\s*$#s',
            '#(<!--.*?-->)|(?<!\>)\s+(<\/.*?>)|(<[^\/]*?>)\s+(?!\<)#s', // t+c || o+t
            '#(<!--.*?-->)|(<[^\/]*?>)\s+(<[^\/]*?>)|(<\/.*?>)\s+(<\/.*?>)#s', // o+o || c+c
            '#(<!--.*?-->)|(<\/.*?>)\s+(\s)(?!\<)|(?<!\>)\s+(\s)(<[^\/]*?\/?>)|(<[^\/]*?\/?>)\s+(\s)(?!\<)#s', // c+t || t+o || o+t -- separated by long white-space(s)
            '#(<!--.*?-->)|(<[^\/]*?>)\s+(<\/.*?>)#s', // empty tag
            '#<(img|input)(>| .*?>)<\/\1>#s', // reset previous fix
            '#(&nbsp;)&nbsp;(?![<\s])#', // clean up ...
            '#(?<=\>)(&nbsp;)(?=\<)#'//, // --ibid
            // Remove HTML comment(s) except IE comment(s)
            //'#\s*<!--(?!\[if\s).*?-->\s*|(?<!\>)\n+(?=\<[^!])#s'
        ),
        array(
            '<$1$2</$1>',
            '$1$2$3',
            '$1$2$3',
            '$1$2$3$4$5',
            '$1$2$3$4$5$6$7',
            '$1$2$3',
            '<$1$2',
            '$1 ',
            '$1'//,
            //""
        ),
    $input);
}
// CSS Minifier => http://ideone.com/Q5USEF + improvement(s)
public static function minify_css($input) {
    if(trim($input) === "") return $input;
    return preg_replace(
        array(
            // Remove comment(s)
            '#("(?:[^"\\\]++|\\\.)*+"|\'(?:[^\'\\\\]++|\\\.)*+\')|\/\*(?!\!)(?>.*?\*\/)|^\s*|\s*$#s',
            // Remove unused white-space(s)
            '#("(?:[^"\\\]++|\\\.)*+"|\'(?:[^\'\\\\]++|\\\.)*+\'|\/\*(?>.*?\*\/))|\s*+;\s*+(})\s*+|\s*+([*$~^|]?+=|[{};,>~+]|\s*+-(?![0-9\.])|!important\b)\s*+|([[(:])\s++|\s++([])])|\s++(:)\s*+(?!(?>[^{}"\']++|"(?:[^"\\\]++|\\\.)*+"|\'(?:[^\'\\\\]++|\\\.)*+\')*+{)|^\s++|\s++\z|(\s)\s+#si',
            // Replace `0(cm|em|ex|in|mm|pc|pt|px|vh|vw|%)` with `0`
            '#(?<=[\s:])(0)(cm|em|ex|in|mm|pc|pt|px|vh|vw|%)#si',
            // Replace `:0 0 0 0` with `:0`
            '#:(0\s+0|0\s+0\s+0\s+0)(?=[;\}]|\!important)#i',
            // Replace `background-position:0` with `background-position:0 0`
            '#(background-position):0(?=[;\}])#si',
            // Replace `0.6` with `.6`, but only when preceded by `:`, `,`, `-` or a white-space
            '#(?<=[\s:,\-])0+\.(\d+)#s',
            // Minify string value
            '#(\/\*(?>.*?\*\/))|(?<!content\:)([\'"])([a-z_][a-z0-9\-_]*?)\2(?=[\s\{\}\];,])#si',
            '#(\/\*(?>.*?\*\/))|(\burl\()([\'"])([^\s]+?)\3(\))#si',
            // Minify HEX color code
            '#(?<=[\s:,\-]\#)([a-f0-6]+)\1([a-f0-6]+)\2([a-f0-6]+)\3#i',
            // Replace `(border|outline):none` with `(border|outline):0`
            '#(?<=[\{;])(border|outline):none(?=[;\}\!])#',
            // Remove empty selector(s)
            '#(\/\*(?>.*?\*\/))|(^|[\{\}])(?:[^\s\{\}]+)\{\}#s'
        ),
        array(
            '$1',
            '$1$2$3$4$5$6$7',
            '$1',
            ':0',
            '$1:0 0',
            '.$1',
            '$1$3',
            '$1$2$4$5',
            '$1$2$3',
            '$1:0',
            '$1$2'
        ),
    $input);
}
// JavaScript Minifier
public static function minify_js($input) {
    if(trim($input) === "") return $input;
    return preg_replace(
        array(
            // Remove comment(s)
            '#\s*("(?:[^"\\\]++|\\\.)*+"|\'(?:[^\'\\\\]++|\\\.)*+\')\s*|\s*\/\*(?!\!|@cc_on)(?>[\s\S]*?\*\/)\s*|\s*(?<![\:\=])\/\/.*(?=[\n\r]|$)|^\s*|\s*$#',
            // Remove white-space(s) outside the string and regex
            '#("(?:[^"\\\]++|\\\.)*+"|\'(?:[^\'\\\\]++|\\\.)*+\'|\/\*(?>.*?\*\/)|\/(?!\/)[^\n\r]*?\/(?=[\s.,;]|[gimuy]|$))|\s*([!%&*\(\)\-=+\[\]\{\}|;:,.<>?\/])\s*#s',
            // Remove the last semicolon
            '#;+\}#',
            // Minify object attribute(s) except JSON attribute(s). From `{'foo':'bar'}` to `{foo:'bar'}`
            '#([\{,])([\'])(\d+|[a-z_][a-z0-9_]*)\2(?=\:)#i',
            // --ibid. From `foo['bar']` to `foo.bar`
            '#([a-z0-9_\)\]])\[([\'"])([a-z_][a-z0-9_]*)\2\]#i'
        ),
        array(
            '$1',
            '$1$2',
            '}',
            '$1$3',
            '$1.$3'
        ),
    $input);
}

public static function removeSlashes($url=''){
    $explode = explode("/",$url);
    $count  = (count($explode) - 2);
    $str = '';
    for($i=0;$i<$count ; $i++){
      $str .=  $explode[$i].'/';
    }
    return $str;
}

 /**
 * this function is for not run all the time the foreach
 */

public static function get_current_currency($all_curencies=[], $field = ""){
       $sesion_currency = \Cookie::get('currency_shop');

      if(!empty($sesion_currency)){
          return $field=='key'? $sesion_currency : @$all_curencies[$sesion_currency][$field];
       }

     $default="";
     if(count((array)$all_curencies)>0){
         foreach ($all_curencies as $k=>$v){
              if(isset($v['default'])){
                $default= $k;
                break;
              }
         }

         $key = !empty($default)? $default : key($all_curencies);

         $return = $field=='key'? $key : $all_curencies[$key][$field] ;

      }else{
        $return = $field=="type"? 'left': "USD";
     }
     $sesion_currency =  null;
     $field  = null;
     $key = null;
   return  $return ;
}

/**
  * Setup the default constants
  */

public static function loadSettingsDefault(){
    $settingsMap = [
          "_theme_name_"=>            ["_THEME_NAME_",           "value",  "not"],
          "_categories_structure_"=>  ["_CATEGORIES_STRUCTURE_", "value1", "yes"],
          "_currencies_"=>            ["_CURRENCIES_",           "value1", "yes"],
          "_active_modules_" =>       ["_ACTIVE_MODULES_",       "value1", "yes"],
          "_attributes_" =>           ["_ATTRIBUTES_",           "value1", "yes"],
          "_main_options_" =>         ["_MAIN_OPTIONS_",         "value1", "yes"],
          "_categories_fields_" =>    ["_CATEGORIES_FIELDS_",    "value1", "yes"],
          "_header_" =>               ["_HEADER_",               "value1", "not"],
          "_footer_" =>               ["_FOOTER_",               "value1", "not"],
          "_shop_sidebar_" =>         ["_SHOP_SIDEBAR_",         "value1", "not"],
          "_lang_view_"=>             ["_LANG_VIEW_",            "value1", "not"],
          "_lang_admin_"=>            ["_LANG_ADMIN_",           "value1", "not"],
          "_footerjs_"=>              ["_FOOTERJS_",             "value1", "not"],
    ];

    try {
        $dbSettings = \DB::table('settings')
                        ->whereIn('param', array_keys($settingsMap))
                        ->orWhere('autoload', 'yes')
                        ->get();
    } catch (\Exception $e) {
        // Handle exception if needed
        $dbSettings = [];
    }

    foreach ($dbSettings as $setting) {
        $param = $setting->param;
        if (isset($settingsMap[$param])) {
            [$constant, $name, $decodeJson] = $settingsMap[$param];
            $value = $decodeJson == "yes" ? (\Wh::isJson($setting->$name) ? json_decode($setting->$name, true) : []) : $setting->$name;
            if (!defined($constant)) {
                define($constant, $value);
            }
        } else {
            $constant = strtoupper($param);
            if (!defined($constant)) {
                $value = \Wh::isJson($setting->value1) ? json_decode($setting->value1, true) : $setting->value1;
                define($constant, $value);
            }
        }
    }

    $dbSettings = null;
}



 public static function getFullLangName($lang=""){
    $lang_file = require "lang.php";
    return $lang_file[$lang];
 }


public static function adminLanguage() {
    $langget = _LANG_VIEW_;
    $ll = '';

    if (\Wh::isJson($langget)) {
        $lang_arr = json_decode($langget, true);

        if (count($lang_arr) > 1) {
            $lang_file = require "lang.php";
            $langAct = @\Input::get('lang');
            $query = @\Input::query();
            unset($query["lang"]);
            $qq = "?";

            if (count($query) > 0) {
                foreach ($query as $k => $q) {
                    $qq .= $k . "=" . $q . "&";
                }
            }

            $ll .= '<div class="col-md-12">
                        <p class="lang_menu">';
            foreach ($lang_arr as $i => $lang) {
                $line = $i == 0 ? "" : "|";
                $class = $langAct == $lang || (empty($langAct) && $i == 0) ? "activLang" : "";

                $ll .= $line . ' <a href="' . $qq . 'lang=' . $lang . '" class="' . $class . '"> <img src="' . url("/resources/assets/langicon/" . strtoupper($lang) . ".png") . '"/> ' . $lang_file[$lang] . '</a> ';
            }

            $ll .= '     <a href="' . url("/admin/general-settings/language") . '" target="_blank"><i class="fa fa-pencil"></i></a>
                        </p>
                    </div>
                    <div class="clear"> </div>';
        }
    }

    return $ll;
}

 public static function defaultLang(){
       $langget = _LANG_VIEW_;
     if(\Wh::isJson($langget)){
        $lang_arr = fromJson($langget);
        return  $lang_arr[0]??'en';
       }
  return $langget ;
 }

 public static function adminLanguageinput(){
    $langAct = @\Input::get('lang');
    if(!empty($langAct))
        return $langAct;


        $langget = _LANG_VIEW_;
     if(\Wh::isJson($langget)){
        $lang_arr = fromJson($langget);
        return  $lang_arr[0]??'en';
       }

   return $langget ;
 }

  public static function langArray($count = ""){
      $langget = _LANG_VIEW_;
     if(\Wh::isJson($langget)){
        $lang_arr = fromJson($langget);
        return !empty($count) ? count($lang_arr) : $lang_arr;
       }

    return !empty($count)?  1 : [$langget];
  }

public static function splitcheck($v="", $lang = "", $p1=""){
  if(!empty($p1))
    $v = self:: splitcheck($v, $p1);

  return (is_array($v) ? (array_key_exists($lang, $v) ? $v[$lang]: "" ) : $v);
 }


 public static function splitlang($text="", $langCode = ""){
    $langMain = LANGMAIN;
    $admin = ISADMINURL;

     if(empty($langCode)){
        $langAct = $admin === "isadmin" ? @\Input::get('lang') : LANGW;
        if(!empty($langAct)){
            $langCode = $langAct;
             }else{
            $langCode = $langMain;
         }
      }
 
     if(\Wh::isJson($text)){
        $textJson = fromJson($text);
        $text     = $textJson[$langCode]??'';
     }else if(is_array($text)){
        $text     = $text[$langCode]??'';
     }
   return $text  ;
 }

 /**
  * Language seted by user is: it
  * Title in database: {"en":"Title product"}
  * if in title does not exits: "it" than will return the value of en
  * \Wh::splitlangfind('{"en":"Title product"}'); 
  * return "Title product" even if the language to website is setted to Italian
  * @param type $text
  * @param type $langCode
  * @return type
  */
  public static function splitlangfind($text="", $langCode = ""){
     $langMain = LANGMAIN;
     $langAct  = !empty($langCode) ? $langCode : LANGW;
     $data     = false;
     
     if(\Wh::isJson($text)){
        $textArray = fromJson($text);
        $data      = true;
     }else if(is_array($text)){
        $data      = true;
        $textArray = $text;
     }

     if($data){
      if(!empty($textArray[$langAct])){
         $text = $textArray[$langAct];
      }else{
         foreach($textArray as $val){
          if(!empty($val)){
             $text = $val;
             break;
          }
         }
       }
     }

    return $text;
  }



 public static function saveField($existing_text="", $text="", $lang = "", $type =""){
     $langget = _LANG_VIEW_;
     $langMain = LANGMAIN;
     if(!\Wh::isJson($langget)){
        return $text;
     }

     if(\Wh::isJson($existing_text)){
        $existing_text = json_decode($existing_text, true);
        @$existing_text[$lang]  = $text;

     }else if(is_array($existing_text)){
         @$existing_text[$lang]  = $text;

     }else{
       $existing_text = $langMain != $lang && !empty($existing_text) ? [$langMain=>$existing_text,$lang=>$text] : [$lang=>$text] ;
     }

    return !empty($type) ?  $existing_text:  toJson($existing_text) ;
 }

 public static function removeKeyObject($object=[]){
     $object = array_values((array)$object);
     return json_decode (json_encode($object), FALSE);
 }

 public static function returnLang(){
      $langSet = LANGW;
      $langMain = LANGMAIN;
      return $langSet != $langMain ?  "/".$langSet : "";
 }


 public static function getDataFile($path = ""){

       try {
        if (file_exists($path)) {
        $inside = "";
        $myfile = fopen($path , "r")  ;
         while(!feof($myfile)) {
           $inside  = $inside.trim(fgets($myfile));
         }
         fclose($myfile);
        return $inside;
        }else{
            return "";
        }
     } catch (\Exception $e) {
        return "";
     }
 }

  public static function calcWidthDefaultImg(){
     try {
       // Aspect ratio
        $aspect_ratio = 0.76;

        $desired_height = intval(mainOption("_media_height",1));
        $container_width = intval(mainOption("_media_width",1));
        // Calculate width corresponding to the desired height
        $width = $aspect_ratio * $desired_height;
        // Calculate width percentage
        $width_percentage = ($width / $container_width) * 100;
        return round($width_percentage, 2);
     } catch (\Exception $e) {
         return "100";
     }
 }

}
