<?php

namespace App\Http\Controllers\Admin;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Modules\Page;
use App\Modules\Categories;

class PageController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    private $pagesType;

     public function __construct() {
         $this->pagesType = \Wh::mergeAllConfig(PAGES_TYPE??[], "pageType");
    }


 public function index($type='',$s='') {
    $page =empty($s)? Page::where('type',$type)->paginate(50) :
    Page::where('type',$type)
        ->where(
          function($query) use ($s) {
                   $query= $query->where("title",'like','%'.$s.'%')
                                 ->orWhere("text",'like','%'.$s.'%')
                                 ->orWhere("cpu",'like','%'.$s.'%');
               return $query;
           })
       ->orderby("id",'desc')
       ->paginate(50);
     
    return view('admin.pages.page', ['page'       => $page, 
                                     'type'       => $type, 
                                     'post_type'  => $this->pagesType, 
                                     'categories' => \Wh::getCategoriesByType($type)]);
 }


  /**
    * Show posts from category
    */

 public function showFromCategory($type='', $catid='') {
    $category = Categories::find($catid);
    $page = $category->pages()
                     ->orderby("id",'desc')
                     ->paginate(50);

     $categories = \Wh::getCategoriesByType($type);

    return view('admin.pages.page', ['page' => $page,'type'=>$type, 'post_type'=>$this->pagesType, 'categories'=>$categories, 'catid'=>$catid ]);
  }
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create($type='',$id="") {
        $page         = Page::where('id','=',$id)->first();
        $json         = fromJson($page->options??'');
        $categoriesIn = !empty($page)? $page->categories()->get()?->pluck('id')->toArray():[];
        
        return view('admin.pages.page-add-edit', ['page'     => $page, 
                                                  'type'     => $type,
                                                  'categoriesIn' => $categoriesIn,
                                                  'postTypes' => $this->pagesType[$type]??[],
                                                  'jsondata' => $json ]);
    }
     /**
     * Enable Disable Block.
     *
     * @return \Illuminate\Http\Response
     */
    public function enableDisable($id="")
    {
        $page = Page::find($id);
        $options = fromJson($page->options);
        $options['enable'] = ($options['enable'] ?? '')=='enable' ? '':'enable';
        Page::where('id',$id)->update(['options'=>toJson($options)]);
        return redirect()->back();
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
    $id    = $request->input('id','');
    $title = $request->input('title','');
    $cpu   = $request->input('cpu','');
    $lang  = $request->input('lang','en');
    $cpu = !empty($cpu)? \Str::slug($cpu, '-') : \Str::slug($title, '-');
    $page = Page::where('id','=',$id)->first();

    if($id != "new" && !empty($id)){
        \Wh::replaceMenuUpdate($page->cpu, $cpu, \Wh::saveField($page->title??'', $title, $lang), $title);
     }

     $metta_decoded = ['metad'=>'','metak'=>'', 'subject'=>''];
      if(!empty($page)){
         $metta_decoded =  fromJson($page->options??'',true);
      }

     $meta = [
             'metad'     => \Wh::saveField($metta_decoded['metad']??'', $request->input('metad',''), $lang, "array"),
             'metak'     => \Wh::saveField($metta_decoded['metak']??'', $request->input('metak',''), $lang, "array"),
             'subject'   =>\Wh::saveField($metta_decoded['subject']??'', $request->input('subject',''), $lang, "array"),
             'enable'    => $request->input('enable',''),
             'template'  => $request->input('template',''),
             'image'     => $request->input('image_main',''),
             'style'     => $request->input('style',''),
             'cssdirect' => $request->input('cssdirect',''),
           ];

      $type  = $request->input('type','');
      $textN = str_replace('contenteditable="true"',"", $request->input('text'));

      $data = [
          'title'   => \Wh::saveField($page->title??'', $request->input('title'), $lang),
          'cpu'     =>  $cpu,
          'type'    =>  $type,
          'options' =>  toJson($meta) ,
          'text'    => \Wh::saveField($page->text??'', $textN, $lang),
          'sort'    =>  $request->input('sort'),
          ];

      \Wh::hooks("save_page");
          
      $object = Page::updateOrCreate(['id' => $id], $data);
      $object->categories()->detach(); 
      $object->categories()->attach($request->input('cat',[]));
      
      session(['status' => _l('Success')]);
      if(!empty(mainOption('_cachehomepage_')) && mainOption('_pageHome_') == $object->id ){
        $contentHome = str_replace( "%-" , "% - " , $contentHome);
        \File::put(base_path('/resources/views/standart/cache/home.blade.php'),\Wh::minify_html($contentHome));
      }
      
      return redirect('/admin/page/add-edit/'.$type.'/'.$object->id."?lang=".$lang) ;
    }

  public function dublicate($id, $lang="", $langto){
    $page = Page::where('id','=',$id)->first();
    
    Page::updateOrCreate(
      ['id' => $id], 
      [
       'title' => \Wh::saveField($page->title??'', \Wh::splitlang($page->title??'', $lang), $langto),
       'text'  => \Wh::saveField($page->text??'', \Wh::splitlang($page->text??'',$lang), $langto),
      ]
    );
   
    return redirect('/admin/page/add-edit/'.$page->type??''.'/'.$id."?lang=".$langto) ;
  }


    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
       Page::destroy($id);
       return redirect()->back();
    }


    /**
     * Load more block.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function loadMore()
    {
          $data = \Wh::get_by_curl(config('app.apiUrl')."bulderblocks");
         if(!empty($data)){
           \Wh::updateSettings("_page_builder_", "", $data, "");
           }
       return view("admin.layouts.ready-blocks");
    }



      /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy_bulk(Request $request, $type='')
    {
            $rowid = $request->input('rowid');
            $action = $request->input('action');
            $s = $request->input('s');
          if(!empty($action) && empty($s)){

              foreach ($rowid as $val){
                 if($action=='del' &&  $val != "1" ){
                         Page::where('id', $val)->delete();
                     }
                 }
           }
       return !empty($s)? redirect('/admin/page/search/'.$type.'/'.$s) :  redirect()->back() ;
    }


}
