<?php

namespace App\Http\Controllers\Admin;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Modules\Admin\Upload;
use Illuminate\Support\Facades\Crypt;

use App\Http\Services\CredentialServices;
use App\Http\Requests\CredentialRequest;
use App\Dto\CredentialDTO;
use App\Http\Services\PaymentService;

class PaymentController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $listModules = \Wh::getSettingsFromJson('_active_modules_');
        $paymentModules = [];
        foreach ($listModules as $v){
          $paymentAdmin = \Wh::config($v, "payment");
          if($paymentAdmin != null)
            $paymentModules[] = $v;
        }

       $rows = (new CredentialServices())->getEnableDescr($paymentModules);
       return view("admin.pages.payment", compact("rows") );
    }

    /**
     * Show the form for creating a new resource.
     *
     * @param string $module The module name
     * @return \Illuminate\Http\Response
     */
    public function settings($module = "")
    {
        // Get credential data for the specified module
        $data = (new CredentialServices())->getFirst($module);

        // Execute 'admin' method of the class dynamically
        $view = (new PaymentService())->getViewAdmin($module, $data);

        // Return view with data
        return view("admin.pages.payment", compact("view", "module", "data"));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(CredentialRequest $request)
    {
       (new CredentialServices())->storeFull($request);
        return redirect()->back();
    }

    /**
     * Enable the module
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function activate($module='', $enable="yes")
    {
      (new CredentialServices())->enable($module, $enable);
       return redirect()->back();
    }

}
