<?php namespace App\Http\Controllers\Admin;

use App\Modules\Categories;
use App\Modules\Product;
use App\Modules\Gallery;
use App\Modules\FileUpload;
use App\Modules\Admin\Upload;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Services\FieldsService;

class ProductsController extends Controller
{
    public function __construct() {
    }
    /**
     *
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
     public function index($s="",$author="") {
          $products = Product::orderBy("id",'desc')->where("hide","!=","10");

         if(!empty($s) && empty($author)){
          $products = $products->where('title','like','%'.$s.'%')
                               ->orWhere('text','like','%'.$s.'%')
                               ->orWhere('description','like','%'.$s.'%') ;
        } elseif(!empty($author)){
          $products = $products->where('user_id','=',$author) ;
        }

        return view('admin.pages.productlist', ['products'       => $products->paginate(50),
                                                'urlis'          => '',
                                                'count_products' => !empty($products)? $products->count() :'0']);
    }


    /**
     * Display a listing of the products.
     *
     * @return \Illuminate\Http\Response
     */
    public function showproductfromcategory($idcat ){
            $category = Categories::find($idcat);
            $products = $category->descendantProducts()
                                ->where("hide","!=","10")
                                ->orderByValue("updated_at");

       return view('admin.pages.productlist', ['products'       => $products->paginate(50),
                                               'id_cat'         => $idcat,
                                               'catname'        => $category->title??'',
                                               'count_products' => !empty($products)? $products->count() :'0']);
    }


    /**
     * Go to product edit or add page so if is new then the id=new
     */

     public function product_add_edit($id="", Request $request) {
         $Uid = \Wh::id_user();
         $product = Product::find($id=="new"? 0 : $id);
         if(empty($product)){
           $product = Product::create(["user_id" => $Uid, "hide" => 10 ]);
         }

         $categories           = $product?->categories()->get();
         $categoriesIn         = !empty($categories)? $categories->pluck('id')->toArray():[];
         $meta                 = $product?->getMetaArray();

         $variations           = $meta->variation??[];     //$product->metaByType("variations")->get();
         $specifications       = $meta->specification??[]; //$product->metaByType("specifications")->get();
         $fieldsVariations     = (new FieldsService())->get('variations');
         $fieldsSpecifications = (new FieldsService())->get('specifications');

         $lang                 = $request->input("lang");

         $product->meta = fromJson($product->meta??'');

         return view('admin.pages.product', ['product'              => $product,
                                             'categoriesIn'         => $categoriesIn,
                                             'variations'           => $variations,
                                             'specifications'       => $specifications,
                                             'fieldsVariations'     => $fieldsVariations,
                                             'fieldsSpecifications' => $fieldsSpecifications,
                                             'meta'                 => $meta,
                                             'lang'                 => $lang,
                                            ]);
    }



    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request){
           $id   = $request->input('id');
           $lang = $request->input('lang');

          $product = Product::find($id);


          $meta = fromJson($product->meta);
          $meta_json = [
                   'metad' => \Wh::saveField($meta['metad']??'', $request->input('metad',''), $lang, "array"),
                   'metak' => \Wh::saveField($meta['metak']??'', $request->input('metak',''), $lang, "array")
                ];




         $variation     = $request->input('variation',[]);
         $specification = \Wh::convertValuesOfArraytoJson($request->input('specification',[]));


         $qtuValues = array_map(function($item) {
                                     return isset($item["qtu"]) ? (int)$item["qtu"] : 0;
                                }, $variation);

          $qtu                  = array_sum($qtuValues) > 0? array_sum($qtuValues) : $request->input('qtu','0');

          $textN                = str_replace('contenteditable="true"',"",$request->input('text',''));
          $descriptionN         = str_replace('contenteditable="true"',"",$request->input('description',''));

          $cpu                  = $request->input('cpu','');
          $cpu                  = \Str::slug($request->input((empty($cpu) ? 'title': 'cpu'),''), '-') ;

          $product->meta        = toJson($meta_json);
          $product->image       = $request->input('image_main');
          $product->stock       = $request->input('stock','');

          $product->title       = \Wh::saveField($product->title, $request->input('title',''), $lang);
          $product->cpu         = $cpu;
          $product->SKU         = $request->input('SKU','');
          $product->price       = number_format($request->input('price','0'), 2, '.', '');
          $product->sale_price  = number_format($request->input('sale_price','0'), 2, '.', '');
          $product->qtu         = $qtu;
          $product->weight      = $request->input('weight'??'0');
          $product->text        = \Wh::saveField($product->text,  $textN, $lang);
          $product->description = \Wh::saveField($product->description, $descriptionN, $lang);
          $product->hide        = 1;
          $product->save();

          $product->updateOrCreateVariations('variation', ['variation'=>$variation]);
          $product->updateMetaOrCreateFromArray(['specification'=>$specification,
                                                 'store'=> $request->input('cpu_store')],false);

          $product->categories()->detach();
          $product->categories()->attach($request->input('cat',[]));
          session(['status' => _l('Success')]);

        return redirect('/admin/product/'.$product->id."?lang=".$lang);
    }


   /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function updatebulk(Request $request)  {
          $action    = $request->input('action');
          $cat       = $request->input('category_id');
          $productid = $request->input('productid');
          $s         = $request->input('s');

          if(!empty($action) && empty($s)){
             foreach ($productid as $val){
                if(!empty($cat) && $action=='move'){
                     $product = Product::find($val);
                     $product->categories()->attach([$cat]);

                }elseif($action=='del'){
                   $this->destroy($val, "nothing");
                }elseif($action=='hide'){
                   Product::where('id', $val)
                           ->update(['hide' => NULL]);
                }elseif($action=='visible'){
                   Product::where('id', $val)
                           ->update(['hide' => 1]);
                }

            }
         }
       return !empty($s)? redirect('/admin/view-products/'.$s) :  redirect()->back() ;
    }

    /**
     *  Upload images for product
     */
    public function upload_images(Request $request, $type="")  {
        $parent_id = $request->input('parent_id');
        $upload    = new Upload();
        $images    = $upload->upload($request);

        if($type=="main"){
           return  \Wh::adminMainImage($images[0]??'');
        }

        foreach($images as $image)
          Gallery::create(['id_post' => $parent_id,'directory' => $image]);

        return \Wh::getAdminGallery($parent_id);
     }



    /**
     *  Upload images for product
     */
    public function remove_image($id=0)  {
        $img     = Gallery::find($id);
        $id_post = $img->id_post;
        @\File::delete(public_path("imgproduct/".$imgs->directory), public_path("imgproduct/thumb_".$imgs->directory));
        $img->delete();
        return \Wh::getAdminGallery($id_post);
    }


  /**
     * Hide product
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function hide_show($id, $action){
       Product::where('id', $id) ->update(['hide'=> $action]);
       return redirect()->back();
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id, $return = ""){
        $product =  Product::find($id);

        $galleries = $product->galleries()->get();
         if(!empty($gallery)){
            foreach($galleries as $gallery){
              @\File::delete(public_path("imgproduct/".$gallery->directory), public_path("imgproduct/thumb_".$gallery->directory));
              Gallery::destroy($gallery->id);
            }
         }
        $product->meta()->delete();
        $product->delete();

       return  empty($return)? redirect()->back():'';
    }



/**
* This is for virtual products, will upload file if product is downloadable
*/
 public function upload_virtual(Request $request){
       $upload     = new Upload();
       $file       = $upload->upload($request, 'product-files/', 'file');
       $file_title = $request->input('field1');
       $product_id = $request->input('parent_id');

       FileUpload::insert(
                          ['id_post'          => $product_id,
                           'file_title'       => $file_title,
                           'file'             => $file[0][0]??'',
                           'originalFileName' => $file[0][1]??'',
                           'date'             => date("Y-m-d h:i:s")
                           ]
                      );

       return  \Wh::getUploadsByIdPost($product_id);
   }

   /**
    * remove product upload file, for downloadable products
    */
  public function remove_file($id){
         $file    = FileUpload::find($id);
         $id_post = $file->id_post;
         \File::delete(public_path("product-files/".$file->file));
         $file->delete();
         return \Wh::getUploadsByIdPost($id_post);
  }

   /**
    * remove product upload file, for downloadable products
    */
  public function getdownloadable($id){
         $get_file = FileUpload::where('id',$id)->first();
         return empty($get_file) ? redirect()->back() :
                 response()->download( public_path('product-files/'.$get_file->file), $get_file->originalFileName) ;
  }


/**
* Incrise Price
*/
 public function increasePrice(Request $request){
        $_additional_price            = $request->input('_additional_price');
        $_additional_type             = $request->input('_additional_type');
        $setting                      = \Wh::getSettingsFromJson("_main_options_");
        $setting['_additional_price'] = $_additional_price;
        $setting['_additional_type']  = $_additional_type;
        \Wh::updateSettings('_main_options_','', toJson($setting) );
       return  redirect()->back();
   }


    /**
     * Move product from one category to other
     */
    public function move_product_to_another_category(Request $request) {
        $from    = $request->get("from");
        $to      = $request->get("to");

        $category = Categories::find($from);
        $products = $category->product()->get();
        foreach ($products as $product) {
          // Detach category each product
          $product->categories()->detach();
          $product->categories()->attach([$to]);
        }
        session(['status' => _l('Total moved products').":".$products->count()]);
        return redirect()->back();

    }

}
