<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Modules\Cart;
use App\Modules\Orders;
use App\Modules\Shipping;
use App\Http\Controllers\Controller;
use App\Dto\UserDTO;
use App\Dto\OrderPaymentDTO;
use App\Http\Services\UserService;
use App\Http\Services\PaymentService;
use App\Http\Services\FieldsService;
use App\Http\Services\CartService;

/**
 * CheckoutController
 *
 * @author Corneli F.
 */
class CheckoutController extends Controller {
    //put your code here

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function checkout() {
            // get total weight of products
            \Config::set('page', 'checkout');

            $cart     = (new CartService())->getCartTotal();
            if(empty($cart->getTotal())){
                 return redirect("/");
            }

            // check if any shipping is setup
            $ifShipping = \Wh::shippingCount();

            $fields = (new FieldsService())->getByType(['billing','shipping']);
            $termAndCondition = mainOption("_checkout_termandcondition_");
            return view('theme::checkout', compact("cart","termAndCondition",'fields','ifShipping'));
    }

    /**
     * Show the form for editing the specified resource.
     * @param Request $request
     * @return array
     */
    public function placeOrder(Request $request) {
             $fieldsService   = new FieldsService();
             $cartService     = new CartService();

             //$fields = $fieldsService->getBoxTypeTitleRequired(['billing','shipping']);

             $billing         = $request->input('billing')??[];
             $shipping        = $request->input('shipping')??[];
             $isShipping      = $request->input('isShipping')??'no';

            //1. check if any required field is empty
             $isEmptyBilling  =  $fieldsService->checkEmpty($billing, 'billing');
             $isEmptyShipping =  $isShipping =="yes" ? $fieldsService->checkEmpty($shipping, 'shipping') : null;

             if (!is_null($isEmptyBilling) || !is_null($isEmptyShipping)) {
               return !is_null($isEmptyBilling) ? $isEmptyBilling : $isEmptyShipping;
             }


        //2. get the country id and check if there is any shipping options
            $countryType    = $isShipping == "yes" ? "shipping" : "billing";
            $countryData    = $isShipping == "yes" ?  $shipping : $billing;
            $countryId      = $fieldsService->getDataByTypeBox('country', $countryType, null, $countryData);
            $shippingId     = $request->input('deliverymethod') ?? 0;
            $if_shipping    = $cartService->checkShipping($countryId);

            if($if_shipping && empty($shippingId)){
                return ['message'=> _l('Please select shipping method'),
                        'javascript' => 'window.location.href="#shipping_method";  $$$(".cl_deliverymethod").each(function() { this.required="required" });'];
            }


            //3. Check if payment method is selected
            $paymentType = $request->payment;
            if (empty($paymentType)) {
                return ['message'=> _l('Please select a payment method')] ;
            }

            //4. if user set the password we register new account or login
             $userData = $fieldsService->getDataByTypeBox(['name','email','password'], 'billing', null, $billing);
             $userDto  = new UserDTO();
             $userDto->setFromArray($userData);

            if($userDto->checkEmailAndPasswordNotEmpty() && !\Auth::check()){
               $signIn = (new UserService())->signInOrSignUp($userDto);
               if ($signIn != "success" || $signIn != "reload") {
                return $signIn;
               }
            }

             // remove password fields from billing
             $billing = $fieldsService->unsetFromArray('password', 'billing', null, $billing);

             // Get cart total with Shipping cost
             $cartDTO = $cartService->getCartTotal($shippingId);

             //5. insert all data in orders
             $shippingData = ['billing'        => $billing,
                              'shipping'       => $isShipping =="yes" ? $shipping : $billing];
             $unicId = \Wh::get_random(50);
             $data  = ['shipping'       => toJson($shippingData),
                       'sessionuser'    => CartService::getSession(),
                       'user_id'        => \Wh::id_user(),
                       'secretnr'       => $unicId, // keep this secret until the payment is done
                       'status'         => 'Incomplete',
                       'currency'       => CURENCY_CODE_KEY,
                       'products_cost'  => $cartDTO->getProductsCost(),
                       'shipping_cost'  => $cartDTO->getShippingCost(),
                       'total'          => $cartDTO->getTotal(),
                       'payd'           => 0,
                       'shipping_id'    => $shippingId,
                       'payment_type'   => $paymentType,
                       'coupon_name'    => $cartDTO->getCouponName(),
                       'coupon_total'   => $cartDTO->getCouponTotal(),
                       'created_at'     => date("Y-m-d H:i:s") ];

                $orderId = Orders::insertGetId($data);

               //6. Update the orderr id and change the typecart for cart
                Cart::where('user_id', CartService::getSession())
                           ->where('typecart', 2)
                           ->update(['orderid' => $orderId]);

               //7. call the selected payment and make the payment
                $orderDto = new OrderPaymentDTO();
                $orderDto->setCheckout($orderId, $cartDTO->getProductsCost(), $cartDTO->getShippingCost(),
                                                  $cartDTO->getTotal(), CURENCY_CODE_KEY, $request->all(), $unicId);

                $paymentData = (new PaymentService())->executeMakePayment($paymentType, $orderDto);

                // Redirect to payment URL if valid, else return the URL
                if (empty($paymentData->url) && $paymentData->message != "ok") {
                    return ['message'=> $paymentData->message];
                } else {
                    return ['message'=> _l('Execute Payment'), 
                            'javascript' =>$paymentData->method=="popup" ? 'popupAjaxCall("'.$paymentData->url .'");' : 
                                                                           'window.location.href="'.$paymentData->url .'";'
                           ];
                }
     }

   /**
    * this method it's mainly used only need to pay, without billing, shipping, registration,..
    * @param Request $request
    * @return array
    */
    public function placeOrderSimple(Request $request)
    {
        // Get the order ID from the request
        $orderId = $request->orderid;

        // Retrieve the order details
        $order = (new Orders())->getOrder($orderId);

        // Check if payment method is selected
        $paymentType = $request->payment;
        if (empty($paymentType)) {
            return _l("Please select a payment method!");
        }

        $orderDto = new OrderPaymentDTO();
        $orderDto->setCheckout($orderId, $order->total, 0, $order->total, CURENCY_CODE_KEY, $request->all(), $order->secretnr);

        $paymentData = (new PaymentService())->executeMakePayment($paymentType, $orderDto);

        $order->payment_type = $paymentType;
        $order->save();

        // Redirect to payment URL if valid, else return the URL
        if (empty($paymentData->url) && $paymentData->message != "ok") {
            return ['message'=> $paymentData->message];
        } else {
            return ['message'=> _l('Execute Payment'), 
                    'javascript' =>$paymentData->method=="popup" ? 'popupAjaxCall("'.$paymentData->url .'");' : 
                                                                   'window.location.href="'.$paymentData->url .'";'
                   ];
        }
    }

  /**
   *  get the list of shipping methods
   * @param Request $request
   * @return string
   */
   public function getShipping(Request $request)
    {
        // Get input values from the request with default values if not provided
        $id       = $request->input('id');
        $kg       = $request->input('kg', 0); // Default to 0 if 'kg' is empty
        $total    = $request->input('total'??0);

        $shipping = Shipping::where(function ($query) use ($id) {
                           $query->where('country', $id)
                                 ->orWhere('country', 0);
                       })
                       ->when($kg, function ($query, $kg) {
                           $query->where(function ($query) use ($kg) {
                               $query->where('weight', '>=', $kg)
                                     ->orWhere('weight', 0);
                           });
                       })
                       ->where('hide_show', '!=', 3)
                       ->orderBy('price', 'asc')
                       ->get();

        $result="";
        if(!empty($shipping)){
          foreach($shipping as $i=>$v){
            $free = !empty($v->free_delivery) ? floatval($v->free_delivery) : 0 ;
            $cost = $v->type_shipping ==3 ? round(($total * $v->price)/100, 2) : \Wh::check_price($v->price, 'none');
            $cost = !empty($free) &&  $total >= $free ? 0 : $cost;
            $price_shipping = $cost > 0 ? priceToHtml($cost, 0, 'not')   : _l("FREE");
            $total_price = $cost + $total;
            $result .= '<li> <label>'
                         . '<input type="radio" autocomplete="off" ' //'.($i==0? 'checked=""':'').'
                              . 'onclick="shipping_calculate(\''.priceToHtml($cost,0,'not').'\', \''.priceToHtml($total_price,0,'not').'\');" value="'.$v->id.'" name="deliverymethod" class="cl_deliverymethod"> '
                         . _lang($v->shipping_name). ' '
                      .'</label><span> '.$price_shipping.'</span></li>';
          }
        }

      return $result ;

  }
}
