<?php
namespace App\Http\Controllers;

use App\Modules\User;
use App\Http\Controllers\Controller;
use App\Dto\UserDTO;
use Laravel\Socialite\Facades\Socialite;
use Laravel\Socialite\Two\FacebookProvider;
use Laravel\Socialite\Two\GoogleProvider;
use Laravel\Socialite\Two\TwitterProvider;
use Laravel\Socialite\Two\TelegramProvider;
use App\Http\Services\CredentialServices;

class LoginSocialMediaController extends Controller {
   private $social  = ['facebook', 'google', 'twitter', 'telegram'];


public function redirectTo($social='')
{
    // redirect
    return $this->configDriver($social)->redirect();
}

private function configDriver($social = 'facebook')
{
    if(!in_array($social, $this->social))
      return redirect()->to('/');

    switch ($social) {
        case 'facebook':
            $provider = FacebookProvider::class;
            break;
        case 'google':
            $provider = GoogleProvider::class;
            break;
        case 'twitter':
            $provider = TwitterProvider::class;
            break;
        case 'telegram':
            $provider = TelegramProvider::class;
            break;
        default:
            throw new \InvalidArgumentException("Unsupported social provider: $social");
    }

    $data = (new CredentialServices()) -> getFirst($social);

    return Socialite::buildProvider(
        $provider,
        [
            'client_id'     => $data->keyp,
            'client_secret' => $data->getKeyS(),
            'redirect'      => url("/signup/social-login-call-back/".$social),
        ]
    );
}

/**
 * Handle back from social media
 *
 */

public function handleCallback($social='')
{
   if(!in_array($social, $this->social))
      return redirect()->to('/');
   
   
    try {
       $user = $this->configDriver($social)->user();
    } catch (\Exception $e) {
       return redirect()->to('/');
    }
    
    // Check if user exists, create if not
    $existingUser = User::where('email', $user->email)->first();

    if ($existingUser) {
        auth()->login($existingUser);
    } else {
         // Create a new user
       $newUser = User::create((new UserDTO($user->name, $user->email))->getArray());
       auth()->login($newUser);
    }
  return redirect()->to('/');
 }
}
