<?php

namespace App\Http\Controllers;
 
use App\Modules\Product;
use App\Modules\Comments; 
use App\Modules\Categories; 
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Modules\Wishlist;
use App\Http\Services\ProductServices;
 

class ProductController extends Controller {
   protected $constructor; 
    
 public function __construct() {  
    }
    
   public function index() {
        //return view('cach.home'); 
     }
   
   public function sort($type='asc') {
         session(['priceSort' => $type]);
         return redirect()->back();
     }
   
    /**
     * Display a listing of the products.
     *
     * @return \Illuminate\Http\Response
     */
    public function showproductfromcategory($params = null, Request $request) {
       
        if(empty($params))
          return view('standart.404'); 

          $array_cpu = explode('/', $params);
          $cpu       = last($array_cpu);
          $category  = Categories::where('cpu', $cpu)->first();
          if(empty($category)){
              return view('standart.404');
           } 

           \Config::set('page', 'productCat'); 
           \Config::set('catID', $category->id);
           \Config::set('maincatID', !empty($category->parent) ? $category->parent : $category->id);

           $products = $category->descendantProducts()
                                //this is macro in AppServiceProvider.php if sort by price is set than will have priority
                                ->orderByValue("id","desc")
                                ->whereMetaArray("attribute",$request->all())
                                ->whereNotIn("hide", [10,0])
                                ->where('qtu','>',0)
                                ->paginate(20);


         $templateName = $request->ajax() ? "theme::productlistAjax" : "theme::productlist";

         return view($templateName, ['products'      => $products, 
                                     'catname'       => $category->title, 
                                     'cat_id'        => $category->id, 
                                     'cat_cpu'       => $category->cpu,
                                     'cattitle'      => $category->title,
                                     'maincat'       =>  '',
                                     'catsStructure' => [] ]);
        }
    
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function search(Request $request) {
            $search = $request->get("s","");
            $cat    = $request->get("cat","");
           
            $searchFor = !empty($search)? $search :"no product much";
            
            \Config::set('page', 'search');   
            \Config::set('search', $search); 
             
            $category = (object)[];
            if(!empty($cat)){
              $category = Categories::where('cpu', $cat)->first();
              \Config::set('catID', $category->id);
              $products = $category->descendantProducts()->where('qtu','>',0);
            }else{
              $products = Product::where('qtu','>',0);
             }
              
              $products = $products->orderBy("id",'desc')
                                   ->whereMetaArray("attribute",$request->all())
                                   ->whereNotIn("hide", [10,0])
                                   //this is macro in AppServiceProvider.php if sort by price is set than will have priority
                                   ->orderByValue("id","desc")  
                                   ->where(function($query) use ($searchFor) {
                                        $query->where('title', 'like', '%'.$searchFor.'%')
                                              ->orWhere('text', 'like', '%'.$searchFor.'%')
                                              ->orWhere('description', 'like', '%'.$searchFor.'%')
                                              ->orWhere('SKU', 'like', '%'.$searchFor.'%');
                                    })
                                   ->paginate(20);
           
            if($request->ajax()){ 
              $template= view()->exists('theme::search') ? 'theme::search' : 'standart.search';
            }else{
              $template=  'theme::productlist'; 
            }
            return view($template, ['products' => $products, 
                                    'catname'  => "Search for:".$searchFor, 
                                    'search'   => $searchFor,
                                    'cat_id'   => $category->id ??'',
                                    'cat_cpu'  => $category->cpu ??'', 
                                    'cattitle' => $category->title ??'' ]);
    }
     
    
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function mini_single($request) {
        $id = $request->input('id');
        $product = (new ProductServices())->getProduct($id, "", false);
        
        if($product == null) 
            return _l("No product found");
          
        $template = view()->exists('theme::layouts.minisingle') 
                        ? 'theme::layouts.minisingle' 
                        : 'standart.product.minisingle';
        
        return view($template, $product);
    }
    
     /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
   public function singleproduct($params = null) {
        // Check if params is empty
        if(empty($params)) {
            return view('standart.404'); // Return 404 view if params is empty
        }

        // Explode params by '/'
        $array_cpu = explode('/', $params);
        $cpu = end($array_cpu); // Get the last element of the array as $cpu
         
        $product = (new ProductServices())->getProduct(0, $cpu);
         
         // If product not found, return 404 view
        if($product == null) {
            return view('standart.404');
        }       
        return view('theme::singleproduct', $product);
   }
    
    /**
     * Show Single product by id.
     *
     * @return \Illuminate\Http\Response
     */
    public function singleproduct_by_id($id="") {
        $product = (new ProductServices())->getProduct($id);
        if($product == null) {
            return view('standart.404');
        } 
        return view('theme::singleproduct', $product);
                       
    }
   
    /**
     * Add a new comment
     * 
     */
    public function addNewComment(Request $request){
        $name = $request->input('comment_author');
        
        $data = [
            'comment_author'       => $request->input('comment_author'),
            'comment_author_email' => $request->input('comment_author_email'),
            'id_post'              => $request->input('id_post'),
            'stars'                => $request->input('stars'),
            'comment'              => $request->input('comment'),
            'comment_author_IP'    => $request->ip(),
            'id_user'              => \Wh::id_user(),
            'status'               => 1,
            'created_at'           => date("Y/m/d h:i:s")
        ];
      
      Comments::insert($data);  
        
      return "<span class='fa_publish'></span>";  
    }
    
    
    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function setupcurrency($currid="") {
        $currid = $currid ==777? "" : $currid;
        setcookie('currency_shop', $currid, time() + (86400 * 100), "/");//Session::put('currency_shop',  $currid);
        return redirect()->back();
    }


   /**
     * Display wishlist.
     *
     * @return \Illuminate\Http\Response
     */
    public function wishlist(Request $request) {
            $product = Product::where('qtu','>',0)
                            ->whereNotIn("hide", [10, 0])
                            ->leftJoin('wishlist', 'wishlist.idproduct', '=', 'product.id')
                            ->where('wishlist.session','=',@\Wh::get_sesion())
                            ->paginate(21);
            
            \Config::set('page', 'wishlist');
            
            $templateName = $request->ajax() ? "theme::productlistAjax" : "theme::productlist";
            return view('theme::productlist', ['products' => $product,'catname'=>_l('Wishlist'),'cattitle'=>'' ]);
    }
    
     /**
     * Add to wishlist.
     *
     * @return \Illuminate\Http\Response
     */
     
    public function AddToWishlist(Request $request) {
          $pid = $request->input('id');
          
         // Update or create a record in the wishlist table
          Wishlist::updateOrInsert(
            ['idproduct' => $pid, 'session' => \Wh::get_sesion()],
            ['types' => "product"]  
          );
           
           return "ok";
      }
    
 
   
}
