<?php
namespace App\Http\Controllers;
 
use App\Modules\Product;
use App\Modules\Settings;
use App\Modules\Fields;
use App\Modules\Categories;
use App\Http\Services\FieldsService;

class ShortCode extends Controller
{

/**
 * Contact form Short code
 * [form id=2]
 */
  public function form($data= []) {
    $id =  $data['id']??'';
    $field = Settings::where("value2",$id)->first();
    $row = fromJson($field->value1);
    $random = \Wh::get_random(10);
    return view("standart.default.ContactForm", compact('row','id','random'));
 }
 
/**
 * Menu Short code
 * [menu id=2 class=topMenu align=center]
 */
 public function menu($data= []) {
    $id   = $data['id'] ?? '';
    $type = $data['type'] ?? 'main';
    $align = !empty($data['align']) ? ' center-text' : '';

    $field = !empty($id) ?  Settings::where("id",$id)->first() : 
                            Settings::where("value2", $type)->orderBy("id","desc")->first();

    if(empty($field)) 
      return "";

    $menu = $field->value1 ??  "";
    $class= !empty($data['class'])? " ".$data['class'] : "";
    $randon_class= \Wh::get_random(10, "yes");
    return '<div class="container_menu_icon" onclick="menu_show_is(this,\'.'.$randon_class.$class.'\')"><div class="bar1"></div><div class="bar2"></div><div class="bar3"></div> </div>
            <ul class="ulmenu '.$randon_class.$class.$align.'"> '.
              \Wh::prepareMenu($menu).
           '</ul>' ;
 }


 /**
 * It will shou double menu if you want to show a menu when user is logged and user is not login than use it
 * [menulogged loggedin=44 loggedout=53]
 */
 public function menulogged($data= []) {
    $loggedin  = $data["loggedin"] ?? '';
    $loggedout = $data["loggedout"] ?? '';
    if(!empty($loggedin) && !empty($loggedout)){
     $field = Settings::where("id",(\Auth::check() ? $loggedin : $loggedout))->first();
    }else{
     $field = Settings::where("param", "_website_menu_")->
                        where("value2", (\Auth::check() ? "topMenuLogin" : "topMenuNOTLogin"))->first();
    }

    if(empty($field)) 
       return "";

    $menu = $field->value1 ?? "";
    $class= !empty($data['class'])? " ".$data['class'] : "";
    $randon_class= \Wh::get_random(10, "yes");
    return '<div class="container_menu_icon" onclick="menu_show_is(this,\'.'.$randon_class.$class.'\')"><div class="bar1"></div><div class="bar2"></div><div class="bar3"></div> </div>
            <ul class="ulmenu '.$randon_class.$class.'">'.
              \Wh::prepareMenu($menu).
           '</ul>' ;
 }

/**
 *  Add to cart Short code, it allow you to add produc to cart by using product ID
 *  [cart title=Add to cart id=21 redirect=cart]
 *  redirect=cart  - it's mean after add to cart will redirect to cart Page
 *  redirect=checkout  - it's mean after add to cart will redirect to checkout Page
 */
public function cart($data= []) {
    $id = $data['id']??'';
    $title = _lang($data['title']??'');
    $random = \Wh::get_random(10, "yes");
    $redirect = !empty($data['redirect']) ? "&redirect=".url('/steps/'.$data['redirect']) : '';

    return   '<a href="#" class="btn btn-big" onclick="cartAjax(\'.'.$random.'\', \'action=AddToCart&id='.$id.'&qtu=1&max_qtu=200&oneQty=yes'.$redirect.'\' ); return false;">
                <span class="'.$random.'">'.$title.'</span>
             </a>';
}


/**
 * Switch currency short code
 * [currency]
 */
public function currency($data= []) {
  return view('standart.default.currency-switch-button');
}

 /**
 * Search form short code
 * [search title=search type=posts]
 * title - placeholder; example: title=Search for products
 * type - posts or products or page; example: type=products
 * mode - ajax or enter; example:  mode=ajax
 * cat - is the category where you want to search only ; example: cat=23
 * attribute - is the class or id of container ajax search; example: attribute=.search
 *
 * [search title=Search type=posts mode=ajax  cat=23 attribute=.search]
 *
 */
public function search($data= []) {
   $title     = $data['title'] ?? _l('Search');
   $type      = $data['type'] ?? '';
   $cat       = !empty($data['cat']) ? '<input type="hidden" name="cat" value="'.$cat.'">': '';
   $url       = $type == "posts" ? _url('/search/posts') : _url('/search/products');
   $mode      = ($data['mode']??'') =='ajax' ? 'ajax' : 'enter' ;
   $attribute = $data['attribute'] ??  '#livesearch' ;
   $type      = null;
  return view('standart.default.searchForm',compact('title' ,'url', 'mode', 'cat', 'attribute'));
}

  /**
 * Get last 10 products from product category where id category equal 8
 * [category id=10]
 * id - product category
 * limit- Limit
 */
public function category($data= []) {
    $id       =  $data['id'] ?? 0;
    $limit    = !empty($data['limit']) ? $data['limit'] : 8;

   if(is_numeric($id)){
    $category = Categories::find($id);
    $products = $category?->product()
                         ->where('product.qtu','>',0)
                         ->where('product.hide', '>' ,0)
                         ->orderBy('product.id','desc')
                         ->paginate($limit);
   }else{
    $products =  Product::where('product.qtu','>',0)
                         ->where('product.hide', '>' ,0)
                         ->orderBy('product.id','desc')
                         ->paginate($limit);
   }



   $view_file = view()->exists('theme::layouts.product')? 'theme::layouts.product' :
                                                            'standart.product.product';
    $view='<div class="productListBlcok">';
        foreach ($products as $product){
           $view .= view($view_file,compact('product'));
        }
    $view .='</div>';
  return $view;
}

 /**
 * Get products where id's products equal 10,15,12
 * [products id=10,15,12]
 * id - product products separated by coma
 */
public function products($data= []) {
    if(empty($data['id']))
        return "";

    $id        = explode(",", $data['id']);
    $products  = Product::where('product.qtu','>',0)
                        ->where('product.hide', '>' ,0)
                        ->whereIn('product.id', $id)
                        ->get();

    $view_file = view()->exists('theme::layouts.product')? 'theme::layouts.product' :
                                                            'standart.product.product';
    $view='<div class="productListBlcok">';
       foreach ($products as $product){
           $view .= view($view_file, compact('product'));
        }
    $view .='</div>';
    return $view;
}

 /**
 * Get main category subcategory, it's good for left sidebar
 * [subcategory]
 */
public function subcategory($data= []) {
    $maincat      = \Config::get('maincatID');
    $catStructure = _CATEGORIES_STRUCTURE_;

    $title = !empty($maincat)? '<h2 class="catTitleLeft">
                                 <a href="'.url(\Wh::returnLang()."/cat/".($catStructure[$maincat]['cpufull']??'')).'">'._lang($catStructure[$maincat]['title']??'').'</a>
                               </h2>': '';
    return '<div class="sidebar-categories sidebarBlock">'._lang($title).'<ul class="categoriesList">'. \Wh::getCatChild().'</ul></div>';
 }

 /**
 * if you need to add additional options for your customers to sort products by color or size or any options  use it
 *
 * [search_filter id=459 title=Color]
 * id - id of variation or specification or specification
 * title - is the title Color or Size or Price range empty
  
 * the variation only for one category and subcategories, that will be the best to add cat id
 */
 public function searchFilter($data= [] ) {
    $id    = $data['id'] ?? '';
    $catid = $data['catid']?? "none" ;
    $title = $data['title']  ?? '';

    $maincat = \Config::get('maincatID');
    if($catid != "none" && $catid != $maincat) return "";

    $field = Fields::find($id) ;
    $suggestions= $field?->suggestions()->get();
    if(empty($suggestions)) return "";
 
    // sho the variations in checkbox
     $li ='<div class="sidebarBlock"> <div class="titleVariation"> '.(!empty($title)? $title : _lang($field->title)).' </div> <div class="variationsList">';

     $li .= \Wh::sugesttionsToCheckboxWithUrl($suggestions, 'param_'.$field->id);

     $li .= '</div> </div>';
     return $li;
}

 /**
   * Return the ads options list
   */
public function variations($data = []){
   $rows = (new FieldsService)->get("variations" ) ;   
   $return   = '';
   foreach ($rows as $row){
        $suggestions = $row->suggestions()->get();
       if(!empty($suggestions)){
          $return .='<div class="sidebarBlock"> <div class="titleVariation">'._lang($row->title).' </div> <div class="variationsList">';
          $return .= \Wh::sugesttionsToCheckboxWithUrl($suggestions, 'param_'.$row->id);
          $return .= '</div> </div>';
       }
  }

  return $return;
}



 public function getChildCat($cat='', $type='', $count=2, $view=''){
    $catStructure = _CATEGORIES_STRUCTURE_;
    $return  = '';
    $catID = \Config::get('catID');
    $selectType = \Config::get('selectType');
    $space = '';
    for($i=0;$i<$count;$i++){
     $space .= '&nbsp;' ;
    }
    $space = $count==1? '' : $space;

    $currentCpuFull = explode("/", ($catStructure[$catID]['cpufull']?? ''));
      if(!empty($catStructure)){
         $catAdmin = CATEGORIES_TYPES;
         $urlBegin=  !empty($catAdmin['product'][$type]) ? 'cat':'categories/'.$type;
        foreach($catStructure as $k=>$v){

              $cpuFull= explode("/", $v['cpufull']);
              if($v['type']==$type && (in_array($cat, $cpuFull) || $cat=='none') && count($cpuFull) == $count){

                $child =  $this->getChildCat($v['cpu'], $type, ( $count +1 ), $view );

                if(!empty($view)){
                    $value = !empty($selectType) ? $k : _url('/'.$urlBegin.'/'.$v['cpufull']) ;
                    $selected = $catID == $k ? ' selected=""':'';
                    $parentbg = !empty($child) ? 'style="background-color:#F0F0F0;"':'';
                    $return .= '<option '.$parentbg.$selected.' value="'.$value.'"> '.$space._lang($v['title']).'</option>';
                    $return .=  $child;
                }else{
                    $class= !empty($child)? 'catWithSubcat':'';
                    $currentParr = in_array($v['cpu'], $currentCpuFull) ? ' currentParrent ':'';
                    $currentCat = $catID == $k ?  ' currentCat':"";
                    $return .= '<li class="'.$class.$currentParr.$currentCat.'"><a href="'._url('/'.$urlBegin.'/'.$v['cpufull']).'">'._lang($v['title']).'</a> '.$child.'</li>';
                 }

               }
        }
      }
    return !empty($return)? (!empty($view) ? $return :($count ==1? $return : '<ul class="subCategoriesList">'. $return .'</ul>')): '';
 }

 /**
 * Get categories
 *
 * [categories title=Product Categories type=product view=select select=simple ]
 */
public function categories($data= [] ) {
   $type         = $data['type']?? 'product';
   $title        = $data['title'] ?? '';
   $catStructure = _CATEGORIES_STRUCTURE_;
   $view         = $data['view'] ?? '';
   $select       = $data['select'] ?? '';
   
   if($view=="select")
       return $this->categoriesSelect($type, $title, $catStructure, $select);


   $title = !empty($title)? '<h2 class="catTitleLeft">'._lang($title).'</h2>': '';
   $return= '<div class="sidebar-categories sidebarBlock"> '._lang($title).'<ul class="categoriesList">';

      if(!empty($catStructure)){
         $return .= $this->getChildCat('none', $type, 1);
       }

   $return .= '</ul></div>';

   return  $return;

}

/**
 * Get categories
 *
 * [categoriesSelect title=Product Categories type=product]
 */
public function categoriesSelect($type='', $title='', $catStructure=[], $select ='') {
 $return= '<select name="cat" class="selectNice" '.(empty($select) ? 'onchange="if (this.value) window.location.href=this.value"' : '').'>';
    \Config::set('selectType', $select);
    $return .= '<option value="">'._l("Category").'</option>';
     if(!empty($catStructure)){
        $return .= $this->getChildCat('none', $type, 1, 'select' );
       }
   $return .= '</select>';

   return  $return;

}


/**
 * Simple nice Button
 *  [button url=# title=SHOP NOW color=black]
 */

public function button($data= [] ) {
  $color = !empty($data['color'])?  $data['color']  : 'white';
  $title =  $data['title']??'';
  $url = !empty($data['base_url'])? $data['base_url'] : $data['url']??'';
  $style = $color !='white'? 'style="color:'.$color.'; border-color:'.$color.';"' : '';

  return '<a class="niceButton" '.$style.' href="'.$url.'">'.$title.'</a>';
}
 
 /**
 * Simple nice Button
 *  [subscribe]
 */

public function subscribe($data= [] ) {
 return view('standart.default.subscribe');
}



 /**
 * language switch s
 *  [language]
 */

public function language($data= [] ) {
   $type = $data['type']??'';

   $text="";

   if(\Wh::langArray("count")>1 ){
     $langArrayFull = $type =="full"? require app_path("Helpers/lang.php"):[];
     $text="<div class='lang-switch'>";

       foreach(\Wh::langArray() as $lang){
        $typeText = $type =="full" ? $langArrayFull[$lang] : "";
        $typeText = $type =="short" ? strtoupper($lang) : $typeText;
        $text = $text.'<a href="/set-lang/'.$lang.'"><img src="'.url("/resources/assets/langicon/".strtoupper($lang).".png").'"/> '.$typeText.'</a> ';
       }

    $text = $text."</div>";

   }
   return $text;
}


  /**
    *  Price sort shortcode [pricesort]
   */
 public function pricesort(){
  $pricemin = \Input::get("pricemin") ?? '';
  $pricemax = \Input::get("pricemax") ?? '';
  return view('standart.default.pricesort', compact('pricemin', 'pricemax'));
 }
 
}
