<?php
namespace App\Http\Controllers;

use App\Modules\User;
use App\Modules\PasswordReset;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Http\Controllers\Controller;
use App\Dto\UserDTO;
use App\Http\Services\UserService;
use App\Modules\Admin\Upload;


class UserController extends Controller {

/**
* this is simple login and registration with redirect
*
* use it like  redirect('/loginPage?url='. \URL::current())
*/
public function simpleLoginPage(){
      if( \Auth::check()){
          $redirect = @e(\Input::get("url"));
          $redirect = !empty($redirect) ? str_replace(url('/'), '', $redirect): '/';
        }
     return \Auth::check() ? redirect($redirect)  : view('theme::myaccount');
 }


/**
* This is for admin page
*/
public function showLoginPage() {
    return \Auth::check()? redirect("/admin")  : view('standart.auth.login');
 }

/**
 * Handle an authentication attempt.
 *
 * @return Response
 */
public function authenticate(Request $request) {
   $credentials = $request->only('email', 'password');
   return (new UserService())->signIn($credentials);
  // return redirect()->intended($_SERVER['HTTP_REFERER'])l
}

/**
 * Handle an registration and authentication attempt.
 *
 * @return Response
 */
public function registration(Request $request){
     $userDTO= new UserDTO($request->input('name'),
                           $request->input('email'),
                           $request->input('password'),
                           true);
     return (new UserService())->registrations($userDTO);
}

/**
 * Resend verification email
 **/
public function resendVerificationEmail(Request $request){
     // Get the email from the request
    $email = $request->input('email');

    // Redirect if email is empty
    if (empty($email)) {
        return redirect("/");
    }

    // Instantiate UserService to handle email verification
    $userService = new UserService();

    // Call the resendVerificationEmail method and get the result
    $message = $userService->resendVerificationEmail($email);

    // Redirect if the message is "redirect"
    if ($message === "redirect") {
        return redirect("/");
    }
   
    $content = "<br/><br/><br/><br/>".
               "<h1 class='text-center greenColor'><i class='fa fa-check-circle'></i> "._l($message)."</h1>";

   return view('theme::pages',["rows" => (object)["title"=>_l("Email Verification")],
                               "content"=>$content]);
 }

/**
* Handle an reset pass.
*
* @return Response
*/
public function resetPassword(Request $request)
{
    // Get the email address from the request
    $email = $request->input('email');
    return (new UserService())->resetPassword($email);
}

  /**
 * Handle an email new password show view.
 *
 * @return Response
 */
public function newPassword($token="")
{
    // If the token is empty, redirect to the homepage
    if(empty($token)) {
        return redirect("/");
    }
    
    // Retrieve the email associated with the token
    $email = PasswordReset::where("token", $token)->first();

    // If the email associated with the token exists, load the reset password view
    if(!empty($email)) {
        return view('standart.auth.resetPassword', compact("token"));
    } else {
        // If no matching email is found for the token, redirect to the homepage
        return redirect("/");
    }
}
  /**
 * Handle an email new password update.
 *
 * @return Response
 */
 public function updateNewPassword(Request $request)
{
    // Retrieve inputs from the request
    $password = $request->input('password');
    $password_repeat = $request->input('password_repeat');
    $token = $request->input('reset_token');

    // Validate if the password is not empty
    if (empty($password)) {
        return _l("Please enter a new password.");
    }

    // Validate if the passwords match
    if ($password != $password_repeat) {
        return _l("Passwords don't match.");
    }

    // Validate if the token is not empty
    if (empty($token)) {
        return _l("Something went wrong.");
    }

    // Find the email associated with the token
    $email = PasswordReset::where("token", $token)->first();

    // If the email associated with the token exists
    if (!empty($email)) {
        // Update the user's password
        User::where('email', $email->email)
            ->update(['password' => bcrypt($password)]);

        // Delete the password reset entry
        PasswordReset::where('id', $email->id)->delete();
    }

    // Return a response to reload the page
    return "reload";
}


  /**
 * Handle an email log out.
 *
 * @return Response
 */
 public function logout(){
   Auth::logout();
   return redirect("/");
 }
 
  /**
 * Handle an email Verification.
 *
 * @return Response
 */
public function emailVerification(Request $request)
{
    // Retrieve the verification token from the request
    $verToken = $request->input('verification_token');

    // If a verification token is provided, update the user's email verification status
    if (!empty($verToken)) {
        User::where('verification_token', $verToken)
             ->update(['verification_token' => "", "email_verified_at" => date("Y-m-d H:i:s")]);
    }

    // Prepare the content to be displayed on the verification page
    $content = '';
    if (!empty($verToken)) {
        // If a verification token is provided, display a success message and redirect after 2 seconds
        $content = "<br/><br/><br/><br/>" .
                   "<h1 class='text-center greenColor'><i class='fa fa-check-circle'></i> " . _l("Your email has been verified") . "</h1>" .
                   "<script type='text/javascript'> window.setTimeout(function(){ window.location.href = '" . url("/") . "'; }, 2000);</script>";
    }

    // Return the 'pages' view with the content to be displayed
    return view('theme::pages', [
        "rows" => (object)["title" => _l("Email Verification")],
        "content" => $content
    ]);
}

  /**
 * Handle an myaccount.
 *
 * @return Response
 */
public function myaccount(Request $request)
{
    // Check if a verification token is provided in the request
    $verToken = $request->input('verification_token');
    if (!empty($verToken)) {
        // If a verification token is provided, update the user's email verification status
        User::where('verification_token', $verToken)
             ->update(['verification_token' => "", "email_verified_at" => date("Y-m-d H:i:s")]);
    }

    // Retrieve user account information if the user is authenticated
    $user = Auth::check() ? User::find(\Wh::id_user()) : null;

    // Return the 'myaccount' view with the user information
    return view('theme::myaccount', ["user" => $user]);
}

  /**
 * Handle an reset pass.
 *
 * @return Response
 */
public function updatemyaccount(Request $request)
{
    // Check if user is authenticated
    if (!Auth::check()) {
        return "Error: You must be logged in to perform this operation";
    }

    // Extract input data
    $cname    = $request->input('cname')??'';
    $name     = $request->input('name')??'';
    $password = $request->input('password');
    // Prepare data for update
    $userData = ['name' => $name, 
                 'cname'=> $cname ];
    
    $icon     = (new Upload())->simpleUploadImg($request, 'userimage','cropsmall', '/uploads/usericon'); 
    if(!empty($icon))
        $userData['icon'] = $icon;
    
    if (!empty($password)) 
        $userData['password'] = bcrypt($password); // Hash the password if provided
     

    // Get the user ID
    $userId = \Wh::id_user();

    // Update user data
    User::where('id', $userId)->update($userData);

    // Optionally update user meta data if required
    // Usermeta::updateOrCreateMeta($userId, $request->meta);

    // Redirect back to the previous page
    return redirect()->back();
}

}
