<?php
namespace App\Http\Services;

use App\Modules\Credential;
use App\Dto\CredentialDTO;
use App\Http\Requests\CredentialRequest;
use App\Modules\Admin\Upload;

class CredentialServices {
    
    public function getList($param=[]) {
       return Credential::whereIn('param',$param)->get();
    }
    
    public function getEnableDescr($param=[]) {
       // Fetch credentials based on the provided parameters
        $credentials = Credential::select('param', 'enable','description')->whereIn('param', $param)->get();
        
        // Initialize an empty array to store enable descriptions
        $rows = [];
    
        // Iterate over each parameter
        foreach ($param as $val) {
            // Find the credential with the matching parameter value
             $credential = $credentials->where('param', $val)->first();
            // If the credential exists, add enable status and description to the result array
               $rows[$val] = [
                    'enable' => !empty($credential) ? $credential->enable : 'not',
                    'description' => !empty($credential) ? $credential->description : ''
                ];
             
        }
    
        return $rows;
    }
    
    public function getFirst($param='') {
       $data = Credential::where('param',$param)->first();
       return new CredentialDTO($data ?? []);
    }
    
   public function enable($param='', $enable=''){
        // Update or create a record
        Credential::updateOrCreate(
            ['param' => $param], // Attributes to match existing record
            ['param' => $param, 'enable'=>$enable]
        );
    }
    
    public function store($param='', $data = []){
        // Update or create a record
        Credential::updateOrCreate(
            ['param' => $param], // Attributes to match existing record
            $data
        );
    }
 
    
    public function storeFull(CredentialRequest $request)
    {
        $requestData = (object) $request->validated();
        
        // Retrieve existing record or create a new one
        $dataDb = $this->getFirst($requestData->param);
        
        // Update fields based on language
        $fieldsToUpdate = ['title', 'description', 'instruction'];
        foreach ($fieldsToUpdate as $field) {
            $requestData->$field = \Wh::saveField($dataDb->$field ?? '', $requestData->$field ?? '', $requestData->lang);
        }
        
        // Create CredentialDTO object
        $data = new CredentialDTO($requestData);
        
        // Upload image if provided
        $data->image = $request->hasFile('image') ? (new Upload())->simpleUpload($request, "image") : ($requestData->img ?? '');
        
        // Update or create the record
        $this->store($data->param, $data->toArray()); 
    }
    
}