<?php

namespace App\Http\Services;
 
use App\Mail\SendMail;
use App\Modules\Page;
use Illuminate\Support\Facades\Mail; 

class EmailService {
  
   public function buildEmail($type = '', $data = [])
    {
        // Extract email from data or set to empty string
        $email = $data['email'] ?? '';
        
        // Find the page based on the provided type
        $page = Page::where('main', $type)->first();
    
        // If page not found or not enabled, return empty string
        if (empty($page)) {
            return '';
        }
    
         $pageOptions = json_decode($page->options, true);
         
         // If page not found or not enabled, return empty string
         if (empty($pageOptions['enable']) || $pageOptions['enable'] !== "enable" || empty($email)) {
            return '';
         }
        
        // Split content and subject based on language
        $content = \Wh::splitlang($page->text);
        $subject = \Wh::splitlang($pageOptions['subject']);
     
        // Find short codes in content and replace with corresponding data
        preg_match_all("/\[([^\]]*)\]/", $content, $matches);
        if (!empty($matches[1])) {
            $arraySearch = [];
            $arrayReplace = [];
            foreach ($matches[1] as $shortCode) {
                if (!empty($data[$shortCode])) {
                    $arraySearch[] = '[' . $shortCode . ']';
                    $arrayReplace[] = $data[$shortCode];
                }
            }
            // Replace short codes in content
            $content = !empty($arraySearch) ? str_replace($arraySearch, $arrayReplace, $content) : $content;
        }
    
        // Send email using Mail facade
        Mail::to($email)->send(new SendMail($subject, $content));
    }  
    
}