<?php
namespace App\Http\Services;
use App\Modules\Fields;
use App\Modules\Shippcountry;
use Illuminate\Http\Request;
use App\Dto\FieldsDTO;
use App\Modules\Suggestions;

class FieldsService {
     
    public function getList($type, $relatedId = null)
    {
        // Create a query builder instance
        $query = Fields::where('type', $type);
    
        // Add a condition for the related ID if provided
        if (!is_null($relatedId)) {
           $query->where('related_id', $relatedId);
        }
        
        $rows =  $query->orderBy('position')->get();
        
        // Return the view with the list of fields
        return compact('type', 'rows', 'relatedId');
    }
    
    public function get($type, $relatedId = null)
    { 
        
        // Create a query builder instance
        $query = Fields::where('type', $type);
    
        // Add a condition for the related ID if provided
        if (!is_null($relatedId) && is_array($relatedId)) {
           $query->whereIn('related_id', $relatedId);
        }else if(!is_null($relatedId) && !is_array($relatedId)){
           $query->where('related_id', $relatedId); 
        }
       return $query->orderBy('position')->get();
    }
    
    public function saveAll(Request $request)
    {
         $ids = $request->input('idField');
         if(empty($ids))
             return "";
         
         // Retrieve the records you want to update
         $records = Fields::whereIn('id', $ids)->get();  
         // Loop through each record and update its attributes
         foreach ($records as $record) {
            $dto = new FieldsDTO();
            $dto->setFromRequest($request, $record->id);
            $record->update($dto->toArray());
         }   
    } 
    
    public function add(Request $request)
    {
        // Instantiate the FieldsDTO object and set its properties from the request
        $dto = new FieldsDTO();
        $dto->setFromRequest($request);
    
        // Insert the data into the database
       $insertedId = Fields::insertGetId($dto->toArray()); // Pass an array of data to the insert method
       return $insertedId;
    }
   
    public function deleteByRelatedIdAndParam($id, $param ="")
    {
      // Insert the data into the database
        Fields::where("related_id",$id)->where("type",$param)->delete(); // Pass an array of data to the insert method
    }
    
    
  public function getBoxTypeTitleRequired($type=[])
    {
      $fields = Fields::whereIn('type',$type)->
                        where("enable","yes")->
                        orderBy('position')->
                        orderBy('type')->get()->groupBy('type');
                        
                        
        $fieldsGrouped = [];

        foreach ($type as $t) {
            $fieldsGrouped[$t] = [];
        
            if (isset($fields[$t])) {
                foreach ($fields[$t] as $field) {
                    $fieldsGrouped[$t][$field->id] = [
                        'boxType' => $field->box_type,
                        'title' => _lang($field->title),
                        'required' => $field->required
                    ];
                }
            }
        }
        
      return (object)$fieldsGrouped;                    
    } 
    
   public function getByType($type=[])
    {
       return Fields::with(['suggestions' => function ($query) use ($type) {
                                 $query->whereIn('type',$type);
                                }])->
                      whereIn('type',$type)->
                      where("enable","yes")->
                      orderBy('position')->
                      orderBy('type')->
                      get()->
                      groupBy('type');
       
         
    }
    
    public function checkEmpty($data=[], $type='', $relatedId = null)
    {
        if (is_null($data) || empty($data)) {
            return null;
        }

        $query = Fields::where('type', $type)->where("enable","yes")->where("required","yes") ;
    
        // Add a condition for the related ID if provided
        if (!is_null($relatedId)) {
           $query->where('related_id', $relatedId);
        }
        $rows =  $query->select('id', 'title')->get();
        
        foreach ($rows as $row) {
            if (empty($data[$row->id])) {
                return [ 'id'=>$row->id, 'message'=>_lang($row->title) . " " . _l("is required") . "!"];
            }
        }
        
        return null;
    }
  
  public function getIdByTypeBox($typeBox='',$type='', $relatedId = null )
    {
        $query = Fields::select('id')->where('type', $type)->where("enable","yes")->where("type_box",$typeBox) ;
        // Add a condition for the related ID if provided
        if (!is_null($relatedId)) {
           $query->where('related_id', $relatedId);
        }
        $row =  $query->first();
        return !empty($row) ? strval($row->id)  : '';
  } 
  
  
   public function unsetFromArray($typeBox='',$type='', $relatedId = null,  $data=[])
    {
       $passwordId =  $this->getIdByTypeBox($typeBox,$type, $relatedId);
        if (!empty($passwordId) && array_key_exists($passwordId, $data)) {
         unset($data[$passwordId]);
        }
        return $data;
   } 
    
    
    public function getDataByTypeBox($typeBox = '', $type = '', $relatedId = null, $from = [])
     {
        $results = [];
        $isArray = is_array($typeBox);
        $typeBoxArray = !$isArray ? [$typeBox] : $typeBox;
        
       // Initialize $results with default values for each type_box
        foreach ($typeBoxArray as $box) {
           $results[$box] = '';
        }
        $query = Fields::where('type', $type)->where("enable", "yes")->whereIn("type_box", $typeBoxArray);
     
        // Add a condition for the related ID if provided
        if (!is_null($relatedId)) {
            $query->where('related_id', $relatedId);
        }
    
        // Execute the query and update $results with retrieved values
        $rows = $query->select('id', 'type_box')->get();
        foreach ($rows as $row) {
          $results[$row->type_box] = $from[$row->id] ?? '';
        }
    
        return $isArray ? $results : ($results[$typeBox] ?? '');
      }
      
      
  /**
  * Convert fields to title fields, like
   *  [1=>"Jon",2=>"Smith",...]  -> ["First Name"=>"Jon", "Last Name"=>=>"Smith",...]
   *  1,2 = fields id from database
  */ 
     public function convert($data =[])
     {
        if (empty($data)) {
            return [];
        }

        $rows = Fields:: whereIn("id", array_keys($data))->select('id','type_box', 'title')->get();
        
        $results = [];
        foreach ($rows as $row) {
          if($row->type_box =="countries" && !empty($data[$row->id])){
            $country = Shippcountry::find($data[$row->id]);
            $results[_lang($row->title)] = $country->country ?? '';
          }elseif($row->type_box =="select"){
           //if the type is select than the value will be the id of suggestion     
            $results[_lang($row->title)] = $row->suggestionsGetTitleById($data[$row->id]??'');
          }elseif($row->type_box =="checkbox"){
            //if is checkbox than may have 10 differents suggestion  
            $results[_lang($row->title)] = is_array($data[$row->id] ?? '') ?
                                              $row->suggestionsGetTitleByArrayIds($data[$row->id]) :
                                              $row->suggestionsGetTitleById($data[$row->id]??'');
          }else{
            //the rest the value is simple string
            $results[_lang($row->title)] = $data[$row->id] ?? '';
          }
        }
        
        return $results;
      }
      
      
       /**
  * Convert to variations
   *  ["key"=> [price => 810.00,
                sku => squss,
                qtu => 16,
                weight => 210.00,
                459 => 8187,
                460 => 8195,]
        ....]  
     TO 
      ["key"=> [price => 810.00,
                sku => squss,
                qtu => 16,
                weight => 210.00,
                459 => 8187,
                460 => 8195,
                variation => "color:red; size:L;...", ]
        ....] 
   *  
  */ 
     public function convertVariations($data =[])
     {
       if (empty($data)) {
            return [];
        }
       
        $rows = Fields::where("type", "variations")->select('id','type_box', 'title')->get();
        
        if(empty($rows)){
            return $data;
        }
        
        $fields = [];
        foreach ($rows as $field) {
           $fields[$field->id]  = $field;
        }
        
        $results = [];
        
        foreach ($data as $key=>$value){
            $variation  =  [];
            foreach ($value as $k=>$v){
                if(is_numeric($k) && isset($fields[$k])){
                 $variation[]= _lang($fields[$k]->title)." : "._lang($fields[$k]->suggestionsGetTitleById($v));
                }
             }
             $value['variation'] = implode('; ', $variation);
            $results[$key] = $value;  
         }
        
        return $results;
      }
      
      public function addtSugesstionsFromArray($idField, $type="", $array =[])
      {
          $data =[];
          foreach ($array as $v)
          $data[]= ['field_id' => $idField, 'parent_id' => 0, 'type' => $type, 'title' => $v ];
          Suggestions::insert($data);
      }
     
}