<?php
namespace App\Http\Services;
use App\Dto\OrderPaymentDTO;
use App\Dto\PaymentResponseDTO;
use App\Modules\Orders;
use App\Modules\Page;
use App\Modules\Cart;
use App\Modules\CouponApplied;
use App\Http\Services\FieldsService;
use App\Http\Services\CartService;
use App\Mail\SendMail;
use Illuminate\Support\Facades\Mail;

class PaymentService {

     /**
     * get View Front
     */
     public function getViewFront($moduleName, $data = null){
       return $this->executeMakePayment($moduleName, $data, "front");
     }

     /**
     * get View Admin
     */
     public function getViewAdmin($moduleName, $data = null){
       return $this->executeMakePayment($moduleName, $data, "admin");
     }

    /**
     * Execute MakePayment
     */
   public function executeMakePayment($moduleName, $data = null, $methodName = "makePayment"){
        $moduleName = cleanString($moduleName);
        // Get the class name from configuration
        $className = \Wh::config($moduleName, "payment");
        // Construct fully qualified class name
        $fullClassName = $className != null ? '\Platform\Modules\\' . $moduleName . '\\' . $className : '';
        // Execute 'admin' method of the class dynamically
        return $this->execute($fullClassName, $methodName, $data);
    }

   /**
     * Execute Method
     *
     * @param string $className The name of the class containing the method
     * @param string $functionName The name of the method to execute
     * @param mixed $data Optional data to pass to the method
     * @return mixed The return value of the executed method, or null if the method cannot be executed
     */
    public function execute($fullClassName, $functionName, $data = null)
    {
        $return = null;

        // Check if $className is not empty and is a string
        if (!empty($fullClassName) && is_string($fullClassName)) {

            // Check if the class exists
            if (class_exists($fullClassName)) {
                // Instantiate the class
                $instanceObj = new $fullClassName;

                // Check if the object was successfully created
                if (is_object($instanceObj)) {
                    // Check if the '$functionName' method exists in the class
                    if (method_exists($instanceObj, $functionName)) {
                        // Call the $functionName method dynamically
                        $return = $data==null ? $instanceObj->{$functionName}() : $instanceObj->{$functionName}($data);
                    }
                }
            }
        }

        return $return;
    }



public function successPayment(OrderPaymentDTO $orderDto){
    $order    = Orders::find($orderDto->orderId);
    $secretNr = \Wh::get_random(20);

    $orderDto->orderUnicId =$secretNr;
    Orders::where('id', $orderDto->orderId)->update($orderDto->toOrderUpdate());

    $user_id= '';
    $cartData = (new CartService())->getCart(2, $orderDto->orderId);
    if(!empty($cartData)){
     foreach($cartData as $v){
        $user_id = $v->user_id;
        $product = $v->productInfo();
        $total   = floatval($product->price??1) * intval($v->qty??1) ;
        Cart::where('id', $v->id)
            ->update( ['typecart'=> 4, 'price'=>$total]);
      }
    }

    CouponApplied::where('user_id', $user_id) ->update( ['order_id'=> $orderDto->orderId ]);

    \Wh::hooks('afterpayment',['orderID'=>$orderDto->orderId]);

    // this email will go to costomer
    $this->email_prepare('processing', $orderDto->orderId);


   // send email to admin
   $adminMail= mainOption("admin_mail");
   if(!empty($adminMail) && contains($adminMail, '@'))
    $this->email_prepare('processing', $orderDto->orderId, $adminMail );

   return new PaymentResponseDTO(url('/steps/viewOrder/'.$orderDto->orderId.'/'.$secretNr));
}



 public function email_prepare($type='', $order_id=0, $to = "" ){

    $page = Page::where('main', $type)->first();
    if(empty($page)) return '';

    $page_json = json_decode(($page->options ?? ''), true);


    if(($page_json['enable'] ?? 'not') != "enable" ) return '';

    $content = \Wh::splitlang($page->text) ;
    $subject = \Wh::splitlang($page_json['subject'] ?? '');

    $email ='';
   if($order_id > 0){
    $order= Orders::find($order_id);

    $fieldsService = new FieldsService();

    $jsonShipping =  json_decode($order->shipping, true);
    $billing      =  $jsonShipping['billing'] ?? [];
    $shipping     =  $jsonShipping['shipping'] ?? [];


    $order_amount = $order->payd.' '.$order->currency;

    $emailFieldId = $fieldsService->getIdByTypeBox('email','billing');

    $email        =  $billing[$emailFieldId] ?? \Wh::getUserInfoById("current", "email");

   $table ='<table style="width: 100%; max-width:1400px;min-width:320px" cellspacing="0">
                <tbody>';

             $cartData = (new CartService())->getCart(4,$order->id);
             foreach($cartData as $v){
              $product = $v->productInfo();
              $table .='
                <tr>
                    <td style="text-align: left; border-bottom:1px solid #e4e4e4; padding:15px;border-spacing: 0px;">
                        <a href="'.productUrl($product->cpu, $product->categoryId).'">
                         '._lang($product->title??'').'
                        </a>';

                $table .=!empty($v->options_id)?
                         '<br /> <i> '.($product->optionTitle??'').'</i>':'';

                $table .='
                </td>
                    <td style="text-align: left; border-bottom:1px solid #e4e4e4; padding:15px;border-spacing: 0px;">
                      '.$v->qty.' <b>x</b> '. $v->price .'  '.$order->currency.'
                    </td>
                 </tr>';
                }
               /*
              $table .='
                 <tr>
                     <td style="text-align: right;padding:5px 15px;font-size: 15px;"><b>Coupon:</b> </td>
                     <td style="text-align: left;padding:5px 15px;font-size: 15px;"><b>Five bax</b></td>
                 </tr>

                 <tr>
                     <td style="text-align: right;padding:5px 15px;font-size: 15px;"><b>Discount:</b> </td>
                     <td style="text-align: left;padding:5px 15px;font-size: 15px;"><b>-38.40 '.($orderDto->currency ?? '').'</b></td>
                 </tr>
                 ';*/


               if(!empty($order->shipping_id)){
                 $table .='
                      <tr>
                         <td style="text-align: right;padding:5px 15px;font-size: 15px;"><b>'. _lang(\Wh::getShippingById($order->shipping_id)).'</b> </td>
                         <td style="text-align: left;padding:5px 15px;font-size: 15px;"><b> '.$order->shipping_cost.' '.$order->currency.'</b></td>
                     </tr>
                     ';
                 }
               $table .='
                 <tr>
                     <td style="text-align: right;padding:5px 15px;font-size: 15px;"><b>Total:</b> </td>
                     <td style="text-align: left;padding:5px 15px;font-size: 15px;"><b> '.$order->payd.' '.$order->currency.'</b></td>
                 </tr>

              </tbody>

        </table>';

$shipping_billing = '';
if(!empty($billing)) {
   $billingConverted  = $fieldsService->convert($billing);
   $shippingConverted = $fieldsService->convert($shipping);
   $shipping_billing= '
      <table style="width: 100%; max-width:800px;min-width:320px" cellspacing="0">
            <tbody>

            <tr>
                <td style="text-align: left; border:1px solid #e4e4e4; padding:15px; vertical-align: top;" >
                  <div style="font-style:italic;font-size: 17px; font-weight: bold; margin-bottom:10px;"> '._l('Billing Address').'</div>
                    ';

                      foreach($billingConverted as $bName=>$bValue){
                        $shipping_billing .='<b>'.$bName.' :'.$bValue.'</b><br />';
                      }

   $shipping_billing .='
                </td>';

   $shipping_billing .='<td style="text-align: left; border:1px solid #e4e4e4; padding:15px; vertical-align: top;">
                   <div style="font-style:italic;font-size: 17px; font-weight: bold; margin-bottom:10px;">'._l('Ship to different address').'</div>
                    ';
                      foreach($shippingConverted as $sName=>$sValue){
                        $shipping_billing .='<b>'.$sName.' :'.$sValue.'</b><br />';
                      }

    $shipping_billing .='</td>
                      </tr>

            </tbody>
      </table>';
  }


  $content = @str_replace('[products]',$table, $content);
  $content = @str_replace('[billing_shiping]', $shipping_billing, $content);
  $content = @str_replace('[order_number]', $order_id, $content);
  $content = @str_replace('[order_amount]', $order_amount, $content);
  $content = @str_replace('[first_name]',  '', $content);
  $content = @str_replace('[last_name]',   '', $content );

  $subject = @str_replace('[order_number]', $order_id, $subject);
  $subject = @str_replace('[first_name]', '', $subject);
  $subject = @str_replace('[last_name]', '', $subject);

  }
  // if is email for orders
    $customerEmail= !empty($to) ? $to : $email;

   // check if email is
   try {
    if(!empty($customerEmail) && strpos($customerEmail, '@') !== false)
       Mail::to($customerEmail)->send(new SendMail($subject, $content));
   } catch (\Exception $e) {
    \Log::error('Email sending failed: ' . $e->getMessage());
    }

 }

}
