<?php

namespace App\Http\Services;

use App\Modules\ProductMeta;
use App\Modules\Categories;
use App\Modules\Product;

/**
 * use App\Http\Services\ProductServices;
 * Description of ProductServices
 *
 * @author Corneli F.
 */
class ProductServices {
    //put your code here
  
  
  public static function updateProductMetaFromArray($productId="", $data=[]) {
        
        foreach ($data as $meta) {
          ProductMeta::updateOrCreate(
              [
                  'product_id' => $productId,
                  'param' => $meta['param']
              ],
              [
                  'type' => $meta['type'],
                  'value' => $meta['value']
              ]
          );
        }
}
  
 

/**
 *  Get product by id or by cpu
 * @param type $productId
 * @param type $cpu
 * @param type $showRelated
 * @return null
 */
public function getProduct($productId="", $cpu="", $showRelated=true) {
       // Find product by cpu and ensure it's not hidden
        $product = Product::where(function ($query) use ($cpu, $productId) {
                                    if (!empty($cpu)) {
                                      $query->where('cpu', $cpu);
                                    } else {
                                      $query->where('id', $productId);
                                    } }) 
                          ->whereNotIn("hide", [10, 0])
                          ->first();

        // If product not found, return 404 view
        if(empty($product)) {
            return null;
        }
 
        // Set configuration values
        \Config::set('productID',    $product->id);
        \Config::set('catID',        $product->getFirstCat()->id ?? '');
        \Config::set('productTitle', $product->title);
        \Config::set('page',         'single');
        
         
         $images                  = $product->galleries()->get();
          
         $product->specifications = (new FieldsService())->convert($product->getMetaByTypeArray("specification")??[]);
         $product->variations     = (new FieldsService())->convertVariations($product->getMetaByTypeArray("variation")??[]);
         $tabs                    = \Wh::productTab($product);
         $related                 = null;
         $sidebar                 = null;
        
         if($showRelated){
          $related                = $this->relatedProducts($product);
          $sidebar                = \Wh::getSettings('product-single','value1');
         }
          
         return compact('product','related','images','sidebar','tabs');
}

/**
 *   Will return the random product from the same category like related products
 *  
 */

 public function relatedProducts($product=null){

      if(empty($product))
          return "";
      
      $category = Categories::find($product->getFirstCat()->id??0); 
      
      if(empty($category))
       return "";
     
      $products = $category->descendantProducts()
                           ->where('product.qtu','>',0)
                           ->whereNotIn('product.hide', [0,10])
                           ->where("id","!=", $product->id)
                           ->inRandomOrder()
                           ->paginate(10);

     $data = '';
     if(!empty($products)){
       $template= view()->exists('theme::layouts.product') ? 'theme::layouts.product' : 'standart.product.product';
       foreach ($products as $product){
          $data .=  view($template,['product'=>$product,"fullIMG"=>'set']) ;
        }
     }
    return $data;
   } 

   
   
   
   
   private  function queryMake($query, $by, $k, $v ) {
      if(contains($v,"-")){ 
        $query->where(function ($query) use ($by, $k, $v){
           $values = explode("-",  $v ); 
           foreach ($values as $val) {  
             $query->orWhere($by, $k)->where('value', $val); 
           }
         });
      }else{
        $query->where($by, $k)->where('value', $v);
       }
       return $query;
  }
  
  
  
  /**
   * this method is used in App\Providers\AppServiceProvider to make whereMetaArray
   * @param type $query
   * @param type $arrayOfQuery
   * @return type
   */  
  public function buildQueryFromArray($query, $by, $arrayOfQuery=[]) {
     
     $data = [];
    foreach ($arrayOfQuery as $key => $value) {
       if (trim($value)!="") {
          $data[str_replace("param_", "", $key)] = $value;
       }
     }
      
    if (empty($data))
      return $query;
     
 
    $metaData = $data;
    unset($metaData['pricemin']);
    unset($metaData['pricemax']);
    unset($metaData['cat']);
    unset($metaData['s']);
    unset($metaData['search']);
    unset($metaData['page']);
     
    //printw($metaData);
       foreach ($metaData as $k => $v) {
           $query->with(['meta' => function ($query) use($by, $k, $v)  {
              $query = $this->queryMake($query, $by, $k, $v);
           }])
           ->whereHas('meta', function ($query) use($by, $k, $v)  {
              $query = $this->queryMake($query, $by, $k, $v);
           });
        }          
       /*
        * This is a bunch of cod what i test it, up it's working one bellow its failed one,
        * if you are good with sql and have a better version please make it work and share it with it to korneliu3@gmail.com, THANK YOY!
        * bellow is some of the variants could be helpfull for you
         
                 
       $query->with(['meta' => function ($query) use ($metaData) {
          foreach ($metaData as $k => $v) { 
            $query->where(function ($query) use ($k, $v){
               $query->where('param', $k)->where('value', $v);

            });
            }  
       }])
       ->whereHas('meta', function ($query) use ($metaData)  {
            foreach ($metaData as $k => $v) {
              $query->where(function ($query) use ($k, $v){
                   $query->where('param', $k)->where('value', $v);
             });
           }
       });

        if($i==0){  
          $query->where('param', $k)->where('value', $v);
       }else {  
          $query->orWhere('param', $k)->where('value', $v);
       }
       $i++;

    foreach ($metaData as $k => $v) {
           $query->with(['meta' => function ($query) use ($k,$v) {
              /*   if(contains($v,"-")){ 
                   $values = explode("-",  $v ); 
                   $query->orWhere('param', $k)->whereIn('value', $values);   
                   
                   $query->where(function ($query) use ($k, $v){
                        $values = explode("-",  $v ); 
                        foreach ($values as $val) {  
                          $query->orWhere('param', $k)->where('value', $val); 
                        }
                   });
                 }else{ 
                     echo "here:".$v;
                     $query->where('param', $k)->where('value', trim($v)); 
                 }  
                  echo "here:".$v;
                  $query->where('param', $k)->where('value', trim($v));
           }]);
    }**/ 
                     /* ->with(['meta' => function ($query) {
                            $query->where('param', '30')->where('value', '3');
                        }])
                        ->whereHas('meta', function ($query) {
                            $query->where('param', 'address')->where('value', 'S6Y');
                        })*/
    
    /*
            $query->whereHas('meta', function ($query) use ($metaData) {
                // Add constraints to filter products based on meta information
                  foreach ($metaData as $k => $v) {
                    if(contains($v,"-")){  
                      $query->where(function ($query) use ($k, $v){
                           $values = explode("-",  $v ); 
                           foreach ($values as $val) {  
                             $query->orWhere('param', $k)->where('value', $val); 
                           }
                      });
                    }else{ 
                       //$query->where('param', $k)->where('value', trim($v)); 
                    }  
                  }  
             });
  
       foreach ($data as $k => $v) {
          if($k != 'pricemin' && $k !='pricemax'){ 
            $query = $query->where(function ($query) use ($k, $v) {
                    $values = explode("-",  $v );
                    foreach ($values as $val) {
                           $paramName  = cleanString($k);
                           $paramValue = cleanString($val);
                        $query->orWhere('attr->specification->'.$paramName, $paramValue )
                              ->orWhere('attr->variation->'.$paramName, $paramValue );
                    }
                
                return $query;
            });
          }
        }*/
   
    if (!empty($data['pricemin'])) {
        $query->where('price', '>=', floatval(e($data['pricemin'])));
    }

    if (!empty($data['pricemax'])) {
        $query->where('price', '<=', floatval(e($data['pricemax'])));
    }
 
    return $query;
  }
   
}
