<?php

namespace App\Http\Services;

use App\Modules\User;
use App\Modules\PasswordReset;
use App\Http\Services\EmailService;
use App\Mail\SendMail;
use Illuminate\Support\Facades\Mail;
use App\Modules\Usermeta;
use App\Dto\UserDTO;


class UserService {

  function signInOrSignUp(UserDTO $userDto){
    if($userDto->checkEmailAndPassword())
         return _l('The password and Email address are required!');

      $user = User::where('email', $userDto->getEmail())->first();
      if(!empty($user)){
        return $this->signIn($userDto->getArrayEmailAndPassword());
      }else{
        $newUser =  User::create($userDto->getArray());
        // send an registration email welcome
        (new EmailService())->buildEmail('new-account', ['email' => $userDto->getEmail(), 'password' => $userDto->getPassword()]);
        auth()->login($newUser);
        return "reload";
     }
  }

  function registrations(UserDTO $userDto){
       $messageVer = _l("We sent you an email, please verify your email").". <a href='".url("/signup/resend-verification-email?email=".$userDto->getEmail())."' class='underLine'>"._l("Resend verification email")."</a>";

       if($userDto->checkEmailAndPassword())
         return _l('The password and Email address are required!');

       $user = User::where('email', $userDto->getEmail())->first();


       if(!empty($user))
          return ($userDto->getEmailVerification() && !empty($user->verification_token) ? $messageVer :
                                         _l("This email is all ready registered, try to login"));


       $newUser = User::create($userDto->getArray());

       // send an registration email welcome
       (new EmailService())->buildEmail('new-account', ['email' => $userDto->getEmail(), 'password' => $userDto->getPassword()]);


       // check if admin want to recive an email when new user is registred
        $sendEmailToAdmin = mainOption("_new_account_registered_") == "yes";
        if($sendEmailToAdmin){
            try {
               $subject = _l('New account registered on').' '.url("/");
               $body    = _l('New user registered on your website with email').' : '.$userReg->getEmail();
               
              Mail::to(mainOption("admin_mail"))
                       ->send(new SendMail($subject, $body));
             }catch (\Exception $e) { }
        }

      // if the email verification is not set then user automatically authenticate
      if($userDto->getEmailVerification()){
        $this->sendVerificationEmail($userDto->getEmail(), $userDto->getVerificationToken());
        return $messageVer;
      }

       auth()->login($newUser);
       return "reload" ;

 }

 function sendVerificationEmail($email="", $token=""){
      (new EmailService())->buildEmail('email-verification', ['email'=>$email,
                                       'email_verification_url'=>url("/signup/email-verification?verification_token=".$token)]);
 }


 function signIn($data = [])
 {
    // Check if email and password are provided and not empty
    if (!isset($data['email'], $data['password']) || empty($data['email']) || empty($data['password'])) {
        return _l("Wrong password or email");
    }

    // Check if email verification is required
    $emailVerificationEnabled = mainOption("_email_verification_") == "yes";

    // Find user by email
    $user = User::where('email', $data['email'])->first();

    // If email verification is enabled and user needs to verify email
    if ($emailVerificationEnabled && $user && !empty($user->verification_token)) {
        return _l("We sent you an email, please verify your email") . ". <a href='" . url("/signup/resend-verification-email?email=" . $data['email']) . "' class='underLine'>" . _l("Resend verification email") . "</a>";
    }

    // Attempt to authenticate user
    if (\Auth::attempt(['password' => $data['password'], 'email' => $data['email']], true)) {
        return 'reload';
    } else {
        // Display error message and offer password reset option
        return '<b style="color:red;">' . _l("Wrong password or email") . ". <a href='#' onclick='modaljs(this, \"show\" ); return false;' data-modal='#login_window'>" . _l("Try to reset your password.") . '</a> </b>';
    }

 }


 public function resendVerificationEmail($email = '')
{
    // Get the user by email
    $user = User::where('email', $email)->first();

    // Default message
    $message = "Email Sent";

    // Check if the user and verification token exist
    if (!empty($user) && !empty($user->verification_token)) {
        // Get the current time
        $dateNow = strtotime(date("Y-m-d H:i:s"));

        // Get the time when the email was last sent
        $dateSent = strtotime($user->email_verified_at);

        // Calculate the difference in seconds
        $diffInSeconds = abs($dateNow - $dateSent);

        // Get the count of verification emails sent
        $uMetaVal = (new Usermeta())->getMetaByKey($user->id, "_verification_email");
        $count = intval($uMetaVal);

        // Check if enough time has passed since the last email and limit the count
        if ($diffInSeconds > 300 && $count < 7) {
            // Send an email with verification link
            (new EmailService())->buildEmail('email-verification', [
                'email' => $email,
                'email_verification_url' => url("/signup/email-verification?verification_token=" . $user->verification_token)
            ]);

            // Update the email_verified_at field to the current time
            User::where('email', $email)->update(["email_verified_at" => date("Y-m-d H:i:s")]);

            // Update the count of verification emails sent
            (new Usermeta())->setMetaValueByKey($user->id, "_verification_email", $count + 1);
        } else {
            // Set message to try again in 5 minutes if conditions are not met
            $message = _l("Try again in 5 minutes");
        }
    } else {
        // Set message to redirect if user or verification token not found
        $message = "redirect";
    }

    return $message;
}


public function resetPassword($email='')
{

    // Validate if the email is not empty
    if (empty($email)) {
        return _l('The email address is required') . '!';
    }

    // Check if a user with the provided email exists
    if (User::where('email', $email)->count() <= 0) {
        return _l("We can't find a user with that email address. Try to create an account") . '!';
    }

    // Generate a random token
    $token = \Str::random(60);

    // Data for updating or creating password reset entry
    $data = [
        'email' => $email,
        'token' => $token
    ];

    // Update or create password reset entry
    $passwordReset = PasswordReset::updateOrCreate(['email' => $email], $data);

    // Generate the password reset link
    $link = url("/signup/new-password/" . $token);

    // Send the password reset email
    (new EmailService())->buildEmail('reset-password', ['email' => $email, 'reseturl' => $link]);

    // Return a success message
    return _l('We have emailed your password reset link') . '<br />';
}

}
