<?php

namespace App\Mail;

use Illuminate\Bus\Queueable;
use Illuminate\Mail\Mailable;
use Illuminate\Queue\SerializesModels;
use App\Modules\EmailSetting; // Import the EmailSetting model
use Illuminate\Support\Facades\Config;
//use App\Mail\SendMail;
class SendMail extends Mailable
{
    use Queueable, SerializesModels;

    /**
     * Create a new message instance.
     *
     * @return void
     */
    public $subject;
    public $text;
    public $attachment;
    public $replyToEmail;
    public $replyToName;
    
    public function __construct($subject, $text, $attachment = null, $replyToEmail=null, $replyToName=null) {
        $this->subject      = $subject ;
        $this->text         = $text ;
        $this->attachment   = $attachment;
        $this->replyToEmail = $replyToEmail;
        $this->replyToName  = $replyToName;
        $this->build();
    }

    /**
     * Build the message.
     *
     * @return $this
     */
    public function build()
    {

      $mailSetting = EmailSetting::first();

      if (!$mailSetting) {
            // Handle the case where email settings are not found
            // You can use fallback values or throw an exception
            // For example, throw new \Exception("Email settings not found");
            return "No email Settins Found, please go to admin page General Settings -> Email Settings";
        }

        // Use fetched email settings to configure SMTP
       /*
       if($mailSetting->driver =='smtp')
        config([
            'mail.mailers.smtp.host'       => $mailSetting->smtp_host,
            'mail.mailers.smtp.port'       => $mailSetting->smtp_port,
            'mail.mailers.smtp.username'   => $mailSetting->smtp_username,
            'mail.mailers.smtp.password'   => $mailSetting->smtp_password,
            'mail.mailers.smtp.encryption' => $mailSetting->smtp_encryption,
        ]);
       */

        // Configure mail driver dynamically
        Config::set('mail.driver',     $mailSetting->driver);
        Config::set('mail.host',       $mailSetting->host);
        Config::set('mail.port',       $mailSetting->port);
        Config::set('mail.username',   $mailSetting->username);
        Config::set('mail.password',   $mailSetting->password);
        Config::set('mail.encryption', $mailSetting->encryption);


      $fromEmail    = $mailSetting->from_address;
      $fromName     = $mailSetting->from_name;
      
      $replyToEmail = !empty($this->replyToEmail) ? $this->replyToEmail : $fromEmail;
      $replyToName  = !empty($this->replyToName) ? $this->replyToName : $fromName;
      
              
      $fromEmail = $fromEmail=="hello@example.com" ? mainOption("admin_mail") : $fromEmail;
      
      
      $mail = $this->subject($this->subject)
                   ->from($fromEmail, $fromName)
                   ->replyTo($replyToEmail, $replyToName)
                   ->view( 'standart.email.email')
                   ->with([ 'subject' => $this->subject,
                            'content' => $this->text,
                    ]);
     
        
        if ($this->attachment) {
            $mail->attach($this->attachment->getRealPath(), [
                'as' => $this->attachment->getClientOriginalName(),
                'mime' => $this->attachment->getClientMimeType(),
            ]);
        }
        
      }


}
