<?php
namespace App\Modules;

use App\Modules\Product;
use App\Modules\Page;
use App\Modules\Meta;
use App\Modules\Fields;
use App\Http\Services\ProductServices;

use Illuminate\Database\Eloquent\Model;

class Categories extends Model {
    protected $table = 'categories';
    protected $guarded = ['id'];
    public $timestamps = true;


   protected static function boot()
    {
        parent::boot();

        // Triggering a method when the model is created
        static::created(function ($model) {
            \Wh::updateSettings('_categories_structure_', "", \Wh::getCatHierarchicalArray(\Wh::getCategoriesByType()) );
        });

        // Triggering a method when the model is updated
        static::updated(function ($model) {
           \Wh::updateSettings('_categories_structure_', "",  \Wh::getCatHierarchicalArray(\Wh::getCategoriesByType()) );
        });

        // Triggering a method when the model is deleted
        static::deleted(function ($model) {
           \Wh::updateSettings('_categories_structure_', "",  \Wh::getCatHierarchicalArray(\Wh::getCategoriesByType()) );
        });
    }


   // Method to retrieve all parent categories
    public function parents()
    {
        $parents = collect(); // Initialize an empty collection to store parent categories

        $category = $this; // Start with the current category

        while ($category->parent) {
            $parents->push($category->parent); // Add the parent category to the collection
            $category = $category->parent; // Move to the parent category
        }

        return $parents->reverse(); // Reverse the collection to get the parents in top-down order
    }
    
    

   // Relationship to retrieve immediate child categories
    public function children()
    {
        return $this->hasMany(Categories::class, 'parent', 'id');
    }

    // Many-to-Many relationship with Product through CategoriesProductPage
    public function product()
    {
        return $this->belongsToMany(Product::class, 'categories_product', 'categories_id', 'product_id');
    }

    public function productQuery()
    {
        return $this->belongsToMany(Product::class, 'categories_product', 'categories_id', 'product_id')->getQuery();
    }


    public function fields($type="")
    {
      return $this->hasMany(Fields::class, 'related_id'  )->where('type', $type);
    }

    public function removeFields($type="")
    {
       $fields = $this->hasMany(Fields::class, 'related_id'  )->where('type', $type)->get();
       // Delete the retrieved fields
        foreach ($fields as $field) {
            $field->suggestions()->delete();
            $field->delete();
        }
    }

      // Recursive method to fetch products from subcategories
   public function descendantProducts() {
      // Get IDs of main category and all its descendant categories
        $categoryIds = $this->children()->pluck('id')->push($this->id);

        if($categoryIds->count() == 1){
          $query= $this->productQuery();
        }else{
           // Retrieve products from the main category and its subcategories
          $query= Product::join('categories_product', 'product.id', '=', 'categories_product.product_id')
                       ->whereIn('categories_product.categories_id', $categoryIds)
                       ->select('product.*')
                       ->distinct();
         }

        return $query;
    }


        // Recursive method to fetch products from subcategories
   public function descendantProductsArray() {
      // Get IDs of main category and all its descendant categories
        $categoryIds = $this->children()->pluck('id')->push($this->id);

        $query= Product::join('categories_product', 'product.id', '=', 'categories_product.product_id')
                        //->leftJoin('product_meta', 'product.id', '=', 'product_meta.product_id')
                        //->where('product_meta.param', '30')->where('product_meta.value', '3')
                        //->where('product_meta.param', 'address')->where('product_meta.value', 'S6Y')
                        ->with(['meta' => function ($query) {
                            $query->where('param', '30')->where('value', '3');
                        }])
                        ->whereHas('meta', function ($query) {
                            $query->where('param', 'address')->where('value', 'S6Y');
                        })
                      /*
                       ->where(function ($query){
                           $query->orWhere('product_meta.param', "30")->where('product_meta.value', "2")
                                 ->orWhere('product_meta.param', "30")->where('product_meta.value', "3");
                        })
                       ->where('product_meta.param', "address")->where('product_meta.value', "S6Y")*/
                       ->whereIn('categories_product.categories_id', $categoryIds)
                       ->select('product.*')
                       ->distinct();

        return $query;
    }


     // Recursive method to fetch products from subcategories
    public function productsWithParam($array=[]) {
      // Get IDs of main category and all its descendant categories
       $query=$this->descendantProducts();
       return (new ProductServices())->buildQueryFromArray($query, $array);
    }

    // Recursive method to fetch products from subcategories
    public function productsWithParamAndExclude($request=[], $exclude=[]) {
      // Get IDs of main category and all its descendant categories
        $query=$this->productsWithParam($request);
        return $this->excludeByID($query,$exclude);
    }


    public function excludeByID($query,$exclude)
    {
       return !empty($exclude) ?  $query->whereNotIn("id", $exclude) : $query;
    }

     
   public function getMeata() {
        // Retrieve data from Meta
         $meta = Meta::where('param', 'category_options')->where('related_id', $this->id)
                       ->select('value')
                       ->first();
      return fromJson($meta->value??'');
   }


// Many-to-Many relationship with Page through CategoriesProductPage
 public function pages()
 {
    return $this->belongsToMany(Page::class, 'categories_page', 'categories_id', 'page_id')->getQuery();
 }
       // Recursive method to fetch products from subcategories
 public function descendantPages() {
      // Get IDs of main category and all its descendant categories
     $categoryIds = $this->children()->pluck('id')->push($this->id);
     if($categoryIds->count() == 1){
        return $this->pages();
     }else{
         // Retrieve products from the main category and its subcategories
        return Page::join('categories_page', 'page.id', '=', 'categories_page.page_id')
                    ->whereIn('categories_page.categories_id', $categoryIds)
                    ->select('page.*')
                    ->distinct()->getQuery();

     }
}



/**
 * Check if category exist cpu(converting title to cpu) then return id;
 * if not return 0;
 */
public function category_exist ($title=""){
    $slug = \Str::slug( $title??'', '-');
    $get = Self::where('cpu',$slug)->select("id")->first();
    return !empty($get)? $get->id : 0;
}


public function getByParent ($parent=0, $type="product"){
   $cat= Categories::select("id", "title", "parent")
                ->where("type",$type);
    if ($parent == 0) {
        $cat->where(function ($query) {
            $query->where('parent', 0)->orWhereNull('parent');
        });
    } else {
        $cat->where('parent', $parent);
    }

        return $cat->get();
}
/**
 * @insertByTitle
 * Simple insert the cat by title
 */
public function insertCatByTitle($title, $parent= "", $type = "product", $image="", $param=""){
    $cpu = \Str::slug(($title??''), '-');
    $cpuExist = Categories::where('cpu',$cpu)->count();
    $cpu= $cpuExist > 0 ? $cpu."-".rand(999,10000) : $cpu;
    $data = [
            'title'   => $title,
            'cpu'     => $cpu,
            'metad'   => $title,
            'metak'   => $title,
            'type'    => $type,
            'image'   => $image,
            'param'   => $param,
           ];
       if(!empty($parent)){
           $data['parent']= $parent;
         }
      $id = Self::insertGetId( $data );

    return $id ;
 }


/**
 * Insert category could be many or only one
 * example Woman > Fashion > Dress > Night Dress
 */
public function insert_categories($cat, $delimit = ">",$type = "product"){
             $explode = explode($delimit, $cat);
             $id = 0;
             $all_id=[];

           foreach($explode as $value){
            $is= $this->category_exist($value);
            if($is == 0){
                $id = $this->insertCatByTitle($value, ($id>0 ? $id : ''));
             }else{
                $id = $is;
            }
            $all_id[] = $id;
        }
      return $all_id ;
 }

}
