<?php

namespace App\Modules;

use Illuminate\Database\Eloquent\Model;
 
use App\Modules\Categories;
use App\Modules\Gallery;
use App\Modules\ProductMeta;
use App\Modules\User;
use App\Modules\Fields;
use App\Dto\CartProductDTO;

class Product extends Model
{
    protected $table   = 'product'; 
    protected $guarded = ['id'];
    public $timestamps = true;
    
    
     // Get ProductMeta 
     public function meta()
      {
        return $this->hasMany(ProductMeta::class, 'product_id', 'id');
      }
      
     public function metaByType($type="")
      {
       return $this->hasMany(ProductMeta::class, 'product_id', 'id')->where("type",$type);
      }
     
     public function user()
      {
        return $this->hasOne(User::class, 'id', 'user_id')->select("email","unicid","cname","name","id")->first();
      }
      
     public function galleries()
      {
        return $this->hasMany(Gallery::class, 'id_post', 'id')->orderBy('id','DESC');
      }  
     
     public function getFirstImg()
      {
        $query = $this->hasOne(Gallery::class, 'id_post', 'id')->orderBy("id","desc")->first();
        return !empty($query) ? $query->directory : '';
      }
      
     public function getFirstCat()
      {
        return $this->belongsToMany(Categories::class, 'categories_product', 'product_id', 'categories_id')->first();
      }  
  
     // Many-to-Many relationship with Category through CategoriesProductPage
     public function categories()
      {
        return $this->belongsToMany(Categories::class, 'categories_product', 'product_id', 'categories_id');
      }
  
      // Many-to-Many relationship with Category through CategoriesProductPage
     public function category()
      {
       return $this->belongsTo(Categories::class, 'categories_product', 'product_id', 'categories_id');
      }
    
     // Many-to-Many relationship with Category through CategoriesProductPage
     public function uploads()
      {
        return $this->hasMany(Uploads::class, 'id_post', 'id');
      }
    
     public function metaGetByParam($param="") 
      {
        return $this->hasOne(ProductMeta::class, 'product_id', 'id')->where('param',  $param)->first();
      }
      
      /**
       * 
       * @param type $param = ["chat","views"] OR "chat"
       * @return string
       */
      public function deleteByParam($param=[]) 
      {
        if(empty($param)) 
            return '';
       
        $this->hasMany(ProductMeta::class, 'product_id', 'id')->whereIn('param',  (!is_array($param) ? [$param] : $param))->delete();
      }
      
     public function countMetaByType($type="variation")
      {
        return $this->hasMany(ProductMeta::class, 'product_id', 'id')->where("type",$type)->count();
      }
    
     public function metaUpdateOrCreate($param="", $value="")
      {
        return $this->hasOne(ProductMeta::class, 'product_id', 'id')->updateOrCreate(
                        ['product_id' => $this->id, 'param' => $param], // condition to find existing record
                        ['value' => $value] // data to update or create
                    ); 
      }
      
     public function metaUpdateOrCreateByTypeParamAtribute($type="", $param="",$attribute="", $value="")
      {
        return $this->hasOne(ProductMeta::class, 'product_id', 'id')->updateOrCreate(
                        ['product_id' => $this->id, 'type' => $type, 'param' => $param, 'attribute' => $attribute], // condition to find existing record
                        ['value' => $value] // data to update or create
                    ); 
      }
      
     public function metaUpdateOrCreateByTypeParam($type="", $param="",  $value="")
      {
        return $this->hasOne(ProductMeta::class, 'product_id', 'id')->updateOrCreate(
                        ['product_id' => $this->id, 'type' => $type, 'param' => $param ], // condition to find existing record
                        ['value' => $value] // data to update or create
                    ); 
      }
    
     public function getMetaArray()
      {
        $query = $this->hasMany(ProductMeta::class, 'product_id', 'id')->get(); 
        $array = [];
        foreach ($query as  $value) {
           if(!empty($value->type) && !empty($value->attribute)){
             $array[$value->type][$value->param][$value->attribute] = (object)["id"=>$value->id, "value"=>$value->value];
           }elseif(!empty($value->type) && empty($value->attribute)){
             $array[$value->type][$value->param]                    = (object)["id"=>$value->id, "value"=>$value->value];
           }else{
             $array[$value->param]                                  = (object)["id"=>$value->id, "value"=>$value->value];  
           }  

        } 
       return (object)$array;
     }
      
     public function getMetaByTypeArray($type="")
      {
         $query = $this->meta()->where("type",$type)->get(); 
         $array = [];
         foreach ($query as $value) {
            $param = $value->param;
            if (!isset($array[$param])) {
              $array[$param] = empty($value->attribute) ? $value->value 
                                                        : [$value->attribute => $value->value];
              continue;
            }

            if (is_array($array[$param])) {
              $array[$param] = !empty($value->attribute) ? $array[$param] + [$value->attribute=>$value->value]
                                                         : array_push($array[$param],$value->value);
            } else {
              $array[$param] = !empty($value->attribute) ? [$array[$param], $value->attribute => $value->value] 
                                                         : [$array[$param], $value->value];
            }
        } 
       return $array;
     }
       
    
     public function updateMetaOrCreateFromArray($array=[], $clearString = true)
      {
       if(!empty($array)) 
           ProductMeta::updateOrCreateFromArray($this->id, $array, $clearString);
      }
      
     public function updateOrCreateVariations($type ="",$array=[])
      {
        if(!empty($array)) 
         ProductMeta::updateOrCreateVariations($this->id, $type, $array);
     }
      
     public function metaGetByParamAndValue($param="", $value="" ) 
      {
        $query = $this->hasOne(ProductMeta::class, 'product_id', 'id')
                      ->where('param',  $param)
                      ->where('value', $value)
                      ->select("id")
                      ->first();
        return !empty($query)  ? $query->id : null;
      }
       
     public function getInfoCart($param="")
      {
           $dto = new CartProductDTO($this);
           
           if(empty($param))
             return $dto;  
               
           $rows = Fields::where("type", "variations")->select('id', 'title')->get();
           if(empty($rows)) 
              return $dto;
            
        
           $fields = [];
           foreach ($rows as $field) {
              $fields[$field->id]  = $field;
           } 
            
           $data = $this->hasMany(ProductMeta::class, 'product_id', 'id')->where("type","variation")->where("param",$param)->get();
           foreach($data as $row){
               if($row->attribute=="qtu" || ($row->attribute=="price" && !empty($row->value) && floatval($row->value) > 0) || $row->attribute=="weight"){
                 $dto->{$row->attribute}= $row->value;
               }elseif(is_numeric($row->attribute) && isset($fields[$row->attribute])){
                    $end = empty($dto->optionTitle) ? "":"; ";
                    $dto->optionTitle .= $end._lang($fields[$row->attribute]->title)." : "
                                             ._l($fields[$row->attribute]->suggestionsGetTitleById($row->value)); 
               }
            }
          
            return $dto;
      }
        
      
      
      
}
