<?php

namespace App\Providers;

use Illuminate\Support\ServiceProvider;
use App\Http\Services\ProductServices;
use Illuminate\Database\Query\Builder;
use Illuminate\Database\Eloquent\Builder as EloquentBuilder;
use Illuminate\Pagination\Paginator;

class AppServiceProvider extends ServiceProvider
{
    /**
     * Register any application services.
     *
     * @return void
     */
    public function register()
    {
        require_once __DIR__ . '/../Helpers/constant.php';
        //require_once __DIR__ . '/../Helpers/helpers.php';
        require_once __DIR__ . '/../Helpers/config.php';
    }

    /**
     * Bootstrap any application services.
     *
     * @return void
     */
    public function boot()
    {
        Paginator::useBootstrap();
        
        // Define the custom macro
        EloquentBuilder::macro('excludeIDS', function ($exclude=[]) {
            $query = $this;
            if(!empty($exclude)){
                $query->whereNotIn("id",$exclude);
            }
            return $query;
        });
        
        // Define the custom macro
        EloquentBuilder::macro('whereMetaArray', function ($by, $array, $exclude=[]) {
            $query = $this;
            if(!empty($exclude)){
                $query->whereNotIn("id",$exclude);
            }
            return (new ProductServices())->buildQueryFromArray($query,$by, $array) ;
        });
        
        
       // Define the custom macro
        EloquentBuilder::macro('orderByValue', function ($value="", $by="DESC") {
            $query = $this;
            
            $price_sort = cleanString(session('priceSort'));

            if(!empty($price_sort) && in_array($price_sort, ['asc', 'desc'])) {
                return $query->orderBy('price', $price_sort);
            }else{
                return $query->orderBy($value, $by);
            }
       }); 
        
        
        \View::addNamespace('theme', base_path() . "/platform/Themes/"._THEME_NAME_);
        \View::addNamespace('assets', base_path() . "/resources/assets/");
    }
}
