<?php

namespace Platform\Modules\Paypal\Controllers;

 
use App\Http\Controllers\Controller;
use App\Http\Services\CredentialServices;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Session;
use App\Modules\Orders;
use App\Dto\OrderPaymentDTO;
use \App\Http\Services\PaymentService;
//use Redirect;

class PaypalController extends Controller {
   
   private $credentials;
   private $url = "https://api-m.paypal.com";
    
   
   public function __construct()
    {
       // get the credentials from db
      $this->credentials = (new CredentialServices())->getFirst("Paypal"); 
      $this->url         = $this->credentials->mode === 'sandbox' ? 'https://api-m.sandbox.paypal.com' : 'https://api-m.paypal.com';
    }
   
    /**
     * 
     * @param type $unicId
     * @return OrderPaymentDTO
     * get order info
     */
   private function getOrder($unicId = 0) 
    {
       $order = Orders::where('secretnr',$unicId)->first();
       $orderDto = new OrderPaymentDTO();
       $orderDto->setCheckout($order->id, $order->products_cost, $order->shipping_cost,
                              $order->total, $order->currency, '', $order->secretnr);
                
        return $orderDto;
    }
   
   /**
    * 
    * @return string
    * get access token
    */
    private function getAccessToken(): string
    {
        $headers = [
            'Content-Type'  => 'application/x-www-form-urlencoded',
            'Authorization' => 'Basic ' . base64_encode($this->credentials?->keyp . ':' . $this->credentials?->getKeyS())
        ];
        
        $response = Http::withHeaders($headers)
                        ->withBody('grant_type=client_credentials')
                        ->post($this->url . '/v1/oauth2/token');

        return json_decode($response->body())->access_token;
    }
 
    /**
     * 
     * @param type $orderId
     * @return type
     * show popup
     */
    public function view($orderId){
       $credential = (new CredentialServices())->getFirst("Paypal");
       $clientId = $credential->keyp;
       $currency = CURENCY_CODE_KEY;
       $data = [];
       return view("Paypal::showMethod", compact('data','clientId','currency','orderId'));  
     }  
    
    
  
   
      
    /**
     * @return string
     * create the payment
     */
    public function create($orderId=0): string
    {
        $orderDTO = $this->getOrder($orderId);
        $id = uuid_create();
        $headers = [
            'Content-Type'      => 'application/json',
            'Authorization'     => 'Bearer ' . $this->getAccessToken(),
            'PayPal-Request-Id' => $id,
        ];

        $body = [
            "intent"         => "CAPTURE",
            "purchase_units" => [
                [
                    "reference_id" => $id,
                    "orderID"      => $orderId,
                    "amount"       => [
                        "currency_code" => $orderDTO->currency,
                        "value"         => number_format($orderDTO->total, 2),
                    ]
                ]
            ]
        ];
        
        $response = Http::withHeaders($headers)
                        ->withBody(json_encode($body))
                        ->post($this->url. '/v2/checkout/orders');

        Session::put('request_id', $id);
        Session::put('order_id', json_decode($response->body())->id);

        return json_decode($response->body())->id;
    }
    
    /**
     * @return mixed
     * here we process the payment 
     */
    public function complete($orderId=0)
    {
        $url = $this->url . '/v2/checkout/orders/' . Session::get('order_id') . '/capture';

        $headers = [
            'Content-Type'  => 'application/json',
            'Authorization' => 'Bearer ' . $this->getAccessToken(),
        ];

        $response     = Http::withHeaders($headers)
                                ->post($url, null);

        $responsbody  = fromJson($response->body());
        $status       = $responsbody ['status']??'none';
        $amountPayd   = 0;
        $url          = "";
        try{
          $amount     = $responsbody['purchase_units'][0]['payments']['captures'][0]["amount"]??[];
          $amountPayd = $amount['value']??0;
        } catch (\Exception $e) {}
         
        if($status == 'COMPLETED'){
             
              $orderDTO = $this->getOrder($orderId);
              $orderDTO->payd          = $amountPayd ;
              $orderDTO->status        = 'processing';
              $orderDTO->paymentStatus = 'completed';
              $orderDTO->message       = 'completed';
              $payment = (new PaymentService())->successPayment($orderDTO);
              $url     = $payment->getPath();
        }
         
        return  toJson(["status"=>$status,'payd'=>$amountPayd, 'url'=>$url]);
    }
   
}
