
/**
 * make ajax GET request 
 *  
 // Usage example:
 doGet("your-url-here", 
    function(response) {
        console.log('Request successful');
        console.log(response);
    }, 
    function(status, statusText) {
        console.error('Error:', status, statusText);
    }
);

 * @param {type} url
 * @param {type} successCallback
 * @param {type} errorCallback
 * @return {undefined}
 */  
function doGet(url, successCallback, errorCallback) {
    var xhttp = new XMLHttpRequest();
    xhttp.onreadystatechange = function() {
        if (this.readyState == 4) {
            if (this.status == 200) {
                // Request completed successfully
                if (successCallback && typeof successCallback === 'function') {
                    successCallback(this.responseText);
                }
            } else {
                // Error occurred during the request
                if (errorCallback && typeof errorCallback === 'function') {
                    errorCallback(this.status, this.statusText);
                } else {
                    console.error('Error:', this.status, this.statusText);
                }
            }
        }
    };
    xhttp.onerror = function() {
        // Network error or other errors occurred
        if (errorCallback && typeof errorCallback === 'function') {
            errorCallback(null, 'Network error occurred');
        } else {
            console.error('Network error occurred');
        }
    };
    xhttp.open("GET", url, true);
    xhttp.send();
}
 

/**
 * Make ajax post request
 // Usage example:
var postData = "param1=value1&param2=value2";
doPost("your-url-here", postData,
    function(response) {
        console.log('Request successful');
        console.log(response['message']);
    },
    function(status, statusText) {
        console.error('Error:', status, statusText);
    }
);
 
 * @param {type} url
 * @param {type} data
 * @param {type} successCallback
 * @param {type} errorCallback
 * @return {undefined}
 */
function doPost(url, data, successCallback, errorCallback) {
    var token= document.head.querySelector("[name=csrf-token]").content;
        data =  data + '&_token='+token;
    
    var xhttp = new XMLHttpRequest();
    xhttp.onreadystatechange = function() {
        if (this.readyState == 4) {
            if (this.status == 200) {
                // Request completed successfully
                if (successCallback && typeof successCallback === 'function') {
                    //successCallback(this.responseText);
                    var response;
                    try {
                        response = JSON.parse(this.responseText);
                    } catch (error) {
                        response = {message:this.responseText}; // Use raw response if parsing fails
                    }
                    successCallback(response);
                }
            } else {
                // Error occurred during the request
                if (errorCallback && typeof errorCallback === 'function') {
                    errorCallback(this.status, this.statusText);
                } else {
                    console.error('Error:', this.status, this.statusText);
                }
            }
        }
    };
    xhttp.onerror = function() {
        // Network error or other errors occurred
        if (errorCallback && typeof errorCallback === 'function') {
            errorCallback(null, 'Network error occurred');
        } else {
            console.error('Network error occurred');
        }
    };
    xhttp.open("POST", url, true);
    xhttp.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
    xhttp.setRequestHeader("X-CSRF-Token", token);
    xhttp.send(data);
}


 function popupAjaxCall(url=""){
        doGet(url, 
           function(response) {
               $$("#smallModal").style.display="block";
               setInnerHTML($$("#loadContentSmallModal"), response);
               //$$("#load_content_mini").innerHTML = response;
               console.log('Request successful');
               //console.log(response);
           }, 
           function(status, statusText) {
               console.error('Error:', status, statusText);
           }
        );
 }
 
 function setInnerHTML(elm, html) {
  elm.innerHTML = html;
  
  Array.from(elm.querySelectorAll("script"))
    .forEach( oldScriptEl => {
      const newScriptEl = document.createElement("script");
      
      Array.from(oldScriptEl.attributes).forEach( attr => {
        newScriptEl.setAttribute(attr.name, attr.value); 
      });
      
      const scriptText = document.createTextNode(oldScriptEl.innerHTML);
      newScriptEl.appendChild(scriptText);
      
      oldScriptEl.parentNode.replaceChild(newScriptEl, oldScriptEl);
  });
}
